------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA U20
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   CLOCK CONTROL LATCH AND DECODER
-- VERSION      :   4.0
------------------------------------------------------------------------
-- FILE NAME    :   CLKCNTRLDECODEV40.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/12/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- DECODES THE 8 BIT LATCHED WORD INTO TWO SEPERATE CLOCK PHASE CONTROL
-- WORDS. EACH WORD CAN BE SET INDEPENDENTLY TO 1 OF 9 CODES THAT AFFECT
-- CLOCK PHASING. CLK1 CONTROLS THE PHASE OF THIS DHE SYSCLOCK. CLK2 
-- CONTROLS THE CLOCK PHASE FOR THE NEXT DHE CONNECTED TO THE SYNC OUTPUT
-- PORT.  
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- DECODING IS ACCOMPLISHED AS A MODIFIED ENUMERATED VALUE SUCH THAT
-- THE RANGE IS CONFINED TO -(4 X 960ps) => 4 X 960ps (WHEN USING A 40MHZ
-- STANDARD MASTER CLOCK SOURCE). THIS ALLOWS A SYNCRONIZED PAIR OF DHE's
-- AN ADJUSTMENT RANGE OF APPROX 16ns. THE CLOCK PHASE CAN ALSO BE INVERTED
-- (BY CROSSING THE SYNC CLOCK WIRING) SO A TOTAL RANGE OF +/- 280 DEGREES
-- OF PHASE IS AVAILABLE. 
------------------------------------------------------------------------
-- REVISION HISTORY :
--    DATE    AUTHOR VERSION  MODIFICATIONS
-- 04/12/2004  PCM     4.0    ORIGINAL VERSION
------------------------------------------------------------------------
-- PORT DEFINITIONS :
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity ClkCntrlDecodeV40 is
  port (SYSCLK    : in    std_logic;
         CLK_CS   : in    std_logic;
         READ     : in    std_logic;
         DATA_IN  : in    std_logic_vector(7 downto 0);
         DATA_OUT : inout std_logic_vector(15 downto 0);
         CLK1D0   : out   std_logic;
         CLK1D1   : out   std_logic;
         CLK1D2   : out   std_logic;
         CLK1D3   : out   std_logic;
         CLK2D0   : out   std_logic;
         CLK2D1   : out   std_logic;
         CLK2D2   : out   std_logic;
         CLK2D3   : out   std_logic
         );
end ClkCntrlDecodeV40;

architecture Behavioral of ClkCntrlDecodeV40 is

  signal ClkCntrlReg      : std_logic_vector(7 downto 0);
  signal Clk1Dec, Clk2Dec : std_logic_vector(3 downto 0);

begin
------------------------------------------------------------------------
-- Output signals
------------------------------------------------------------------------
  CLK1D0 <= Clk1Dec(0);
  CLK1D1 <= Clk1Dec(1);
  CLK1D2 <= Clk1Dec(2);
  CLK1D3 <= Clk1Dec(3);
  CLK2D0 <= Clk2Dec(0);
  CLK2D1 <= Clk2Dec(1);
  CLK2D2 <= Clk2Dec(2);
  CLK2D3 <= Clk2Dec(3);

------------------------------------------------------------------------
-- Catch the load value to set the clock generator.
------------------------------------------------------------------------
  ClkCntrlLatch : process(SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (CLK_CS = '1' and READ = '0') then
        ClkCntrlReg <= DATA_IN;
      end if;
    end if;
  end process;

  DATA_OUT <= (x"00" & ClkCntrlReg) when (CLK_CS = '1' and READ = '1') else (others => 'Z');

------------------------------------------------------------------------
-- Decode the latched data
------------------------------------------------------------------------
  Clk1Decode : process(ClkCntrlReg)
  begin
    case ClkCntrlReg(3 downto 0) is
      when "0000" => Clk1Dec <= "0101";
      when "0001" => Clk1Dec <= "0110";
      when "0010" => Clk1Dec <= "1000";
      when "0011" => Clk1Dec <= "1001";
      when "0100" => Clk1Dec <= "1010";
      when "0101" => Clk1Dec <= "1010";
      when "0110" => Clk1Dec <= "1010";
      when "0111" => Clk1Dec <= "1010";
      when "1000" => Clk1Dec <= "0000";
      when "1001" => Clk1Dec <= "0000";
      when "1010" => Clk1Dec <= "0000";
      when "1011" => Clk1Dec <= "0000";
      when "1100" => Clk1Dec <= "0000";
      when "1101" => Clk1Dec <= "0001";
      when "1110" => Clk1Dec <= "0010";
      when "1111" => Clk1Dec <= "0100";
      when others => Clk1Dec <= "0000";
    end case;
  end process;

  Clk2Decode : process(ClkCntrlReg)
  begin
    case ClkCntrlReg(7 downto 4) is
      when "0000" => Clk2Dec <= "0101";
      when "0001" => Clk2Dec <= "0110";
      when "0010" => Clk2Dec <= "1000";
      when "0011" => Clk2Dec <= "1001";
      when "0100" => Clk2Dec <= "1010";
      when "0101" => Clk2Dec <= "1010";
      when "0110" => Clk2Dec <= "1010";
      when "0111" => Clk2Dec <= "1010";
      when "1000" => Clk2Dec <= "0000";
      when "1001" => Clk2Dec <= "0000";
      when "1010" => Clk2Dec <= "0000";
      when "1011" => Clk2Dec <= "0000";
      when "1100" => Clk2Dec <= "0000";
      when "1101" => Clk2Dec <= "0001";
      when "1110" => Clk2Dec <= "0010";
      when "1111" => Clk2Dec <= "0100";
      when others => Clk2Dec <= "0000";
    end case;
  end process;
end Behavioral;
