------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC BOARD
-- DEVICE ID    :   GENERIC FPGA
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   INDICATION LED LOGIC
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   LEDLATCHV40.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/06/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :  PRODUCES A SIGNAL SUITABLE FOR DRIVING THE FRONT PANEL LED. 
-- THE INDICATOR CAN BE STIMULATED VIA ONE OF FOUR INPUTS TO PRODUCE A STEADY
-- OR FLASH INDICATION.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
-- 05/06/2004  PCM   4.0  ORIGINAL VERSION
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK       : SYSTEM CLOCK
-- LED_CS       : SELECT THIS MODULE FOR READ / WRITE
-- READ         : READ OR WRITE_N WHEN CS TRUE
-- RESET : PRODUCES A STEADY INDICATION
-- LED_SRC_1    : ONE SHOT FLASH WHEN THIS SIGNAL GOES HIGH
-- LED_SRC_2    : ONE SHOT FLASH WHEN THIS SIGNAL GOES HIGH
-- LED_SRC_3    : ONE SHOT FLASH WHEN THIS SIGNAL GOES HIGH
-- DATA_IN      : DATA TO SELECT WHICH STIMULUS IS ACTIVE
-- DATA_OUT     : READ BACK FOR PAN 
-- LED_EN       : OUTPUT TO ACTUAL LED DEVICE.
-- 
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity LedLatchV40Pix is
  port (SYSCLK    : in  std_logic;
        LED_CS    : in  std_logic;
        READ      : in  std_logic;
        RESET     : in  std_logic;
        LED_SRC_1 : in  std_logic;
        LED_SRC_2 : in  std_logic;
        LED_SRC_3 : in  std_logic;
        DATA_IN   : in  std_logic_vector(2 downto 0);
        DATA_OUT  : out std_logic_vector(15 downto 0);
        LED_EN    : out std_logic
        );
end LedLatchV40Pix;

architecture Behavioral of LedLatchV40Pix is

  constant LED_PERIOD        : std_logic_vector(19 downto 0) := x"F4240";  -- 25 ms pulse width.
  signal   LedReg            : std_logic_vector(2 downto 0);
  signal   LedEnable         : std_logic_vector(2 downto 0);
  signal   LedLock           : std_logic_vector(2 downto 0);
  signal   LedCntr           : std_logic_vector(19 downto 0);
  signal   LedFlag, LedPulse : std_logic;
begin

------------------------------------------------------------------------
-- Ya' gotta catch the data value to set the the bits baby!
------------------------------------------------------------------------
  LedLatchProc : process(RESET, SYSCLK)
  begin
    if RESET = '1' then
      LedReg <= (others => '0');
    elsif (rising_edge(SYSCLK)) then
           if (LED_CS = '1' and READ = '0') then
             LedReg <= DATA_IN;
           end if;
    end if;
  end process;

  LedEnable(0) <= LedReg(0) and LED_SRC_1;
  LedEnable(1) <= LedReg(1) and LED_SRC_2;
  LedEnable(2) <= LedReg(2) and LED_SRC_3;

  LedEnableProc : process(SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (LedEnable(0) = '1' and LedLock(0) = '0') then
        LedFlag <= '1';
      elsif (LedEnable(1) = '1' and LedLock(1) = '0') then
        LedFlag <= '1';
      elsif (LedEnable(2) = '1' and LedLock(2) = '0') then
        LedFlag <= '1';
      else
        LedFlag <= '0';
      end if;
      LedLock <= LedEnable;
    end if;
  end process;

  LedFlashProc : process(RESET, SYSCLK)
  begin
    if RESET = '1' then
      LedCntr <= (others => '0');
    elsif (rising_edge(SYSCLK)) then
      if (LedFlag = '1') then
        LedCntr  <= (others => '0');
        LedPulse <= '1';
      else
        if (LedCntr < LED_PERIOD) then
          LedCntr <= LedCntr + 1;
        else
          LedPulse <= '0';
        end if;
      end if;
    end if;
  end process;

  LED_EN   <= RESET or LedPulse;
  DATA_OUT <= (x"000" & '0' & LEDReg) when (LED_CS = '1' and READ = '1') else (others => 'Z');

end Behavioral;
