------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA U20
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   SEQUENCER COMMAND DECODER
-- VERSION      :   4.5 
------------------------------------------------------------------------
-- FILE NAME    :   PIXCMDDECODEV45.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   10/23/2005
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :  RECEIVES 32 BIT WORDS FROM VIA THE FPGA INTERFACE, DECODES
-- THE COMMAND AND ASSEMBLES THE DATA STRUCTURE TO CONTROL THE INNNER FUNCTIONS
-- OF THE PIXEL FPGA. WHERE AS THE PAN DECODE MODULE IN THE SEQUENCER FPGA NEEDS
-- TO DEAL WITH ONE AND TWO WORD COMMANDS, THIS MODULE ONLY DECODES AND SEQUENCES
-- SIMPLE SINGLE WORD COMMANDS, I.E 32 BIT READS, 16 BIT WRITES. 
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- ADDRESS RANGES:
-- SERIAL CONFIG PORT    0x0200 => 0x027F    (16 BITS READ / WRITE - WRITE TO 0x027F TRIGGERS WRITE TO EEPROM)
-- SERIAL CONFIG FETCH   0x0280              (3 BITS READ / WRITE - TRIGGERS A READ CYCLE OF EEPROM)
-- CLOCK PHASE PORT      0x0281              (8 BITS READ / WRITE)
-- LED PORT              0x0282              (3 BIT READ / WRITE)
-- SYNC LINK PORT        0x0283              (16 BITS READ / WRITE)
-- SERIAL LINK PORT      0x0284              (16 BITS READ / WRITE) (NOT IMPLEMENTED)
-- EXTERNAL DATA PORT    0x0286              (4 BITS READ / WRITE)
-- FPGA DEBUG REG 1      0x0288              (32 BITS READ). |
-- FPGA DEBUG REG 2      0x0289              (32 BITS READ).  \
-- FPGA DEBUG REG 3      0x028A              (32 BITS READ).  / FUNCTION DEPENDENT ON DEBUG CODE VERSION
-- FPGA DEBUG REG 4      0x028B              (32 BITS READ). |
-- DAC CHANNEL PORT      0x02A0 => 0x02A7    (16 BITS READ / WRITE)
-- ADC CHANNEL PORT      0x02C0 => 0x02C7    (16 BITS READ / WRITE)
-- ADC CHANNEL MODE PORT 0x02C8 => 0x02CF    (4 BITS READ / WRITE)
-- PIX FPGA CONTROL REG  0x02FC              (16 BITS READ / WRITE)
-- STATUS PORT           0x02FD              (16 BITS READ ONLY) (NOT IMPLEMENTED)
-- FIRMWARE ID REGISTER  0x02FF              (16 BITS READ ONLY)
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
-- 04/17/2004  PCM   4.0  ORIGINAL VERSION
-- 08/14/2004  pcm   4.1  added select strobes for fpdp word counter registers.
-- 02/14/2005  PCM   4.4  ADDED CONTROL REGISTER CMD STROBE + DEBUG REGISTER SUPPORT.
-- 10/23/2005  PCM   4.5  ADDED FIRMWARE IDENTITY REGISTER + DATA_WIDTH SIGNAL FOR MUX
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK       : SYSTEM CLOCK
-- RESET        : SYSTEM RESET
-- CMD_STROBE   : CMD_DATA FROM THE FPGA INTERFACE IS VALID
-- CMD_DATA     : 32 BIT COMMAND WORD
-- CFG_CMD_STB  : COMMAND STROBE TO SERIAL CONFIGURATION MODULE
-- CFG_ADDRESS  : ADDRESS OF CONFIGURATION MEMORY TO READ
-- ADC_CMD_STB  : COMMAND STROBE TO READ AN ADC CHANNEL
-- DAC_CMD_STB  : COMMAND STROBE TO WRITE A DAC VALUE
-- ADCDAC_DATA  : DATA OUTPUT TO DAC CHANNELS
-- ADCDAC_ADDR  : ADDRESS OF DAC CHANNEL TO UPDATE
-- EXD_CMD_STB  : COMMAND STROBE TO WRITE AN EXTERNAL DATA NIBBLE
-- EX_DATA      : DATA OUT TO EXTERNAL DATA LATCH.
-- CLK_CMD_STB  : COMMAND STROBE TO SET THE CLOCK GENERATOR PHASE BITS
-- CLK_DATA     : DATA OUT TO CLOCK PHASE LATCH
-- FPDP_CMD_STB : CMD STROBE TO ACCESS THE DEBUG REGISTER SET IN THE FPDP MODULE
-- CNTL_CMD_STB : CMD STROBE TO THE LOCAL (PIXEL FPGA) CONTROL REGISTER
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity PixCmdDecodeV461 is
  port (SYSCLK       : in    std_logic;
        RESET        : in    std_logic;
        CMD_RDY      : in    std_logic;
        CMD_ACK      : out   std_logic;
        CMD_READ     : in    std_logic;
        CMD_DATA     : in    std_logic_vector(31 downto 0);
        ADDRESS      : out   std_logic_vector(6 downto 0);
        REG_DATA     : inout std_logic_vector(15 downto 0);
        DATA_OUT     : out   std_logic_vector(15 downto 0);
        READ_REQ     : out   std_logic;
        READ_ACK     : in    std_logic;
        READ_SYNC    : out   std_logic;
        READ_WRITE_N : inout std_logic;
        CFG_CMD_STB  : out   std_logic;
        ADC_CMD_STB  : out   std_logic;
        MOD_CMD_STB  : out   std_logic;
        DAC_CMD_STB  : out   std_logic;
--           EXD_CMD_STB  : out std_logic;
        SYN_CMD_STB  : out   std_logic;
        CLK_CMD_STB  : out   std_logic;
        FCH_CMD_STB  : out   std_logic;
        LED_CMD_STB  : out   std_logic;
        DBL_WRD_XFER : out   std_logic;
        SIM_ENABLE   : out   std_logic;
        SIM_PIXROWS  : out   std_logic_vector(15 downto 0);
        SIM_PIXCOLS  : out   std_logic_vector(15 downto 0);
        SIM_PIXTIME  : out   std_logic_vector(7 downto 0);
        SIM_CHANNELS : out   std_logic_vector(6 downto 0);
        SIM_DELAY    : out   std_logic_vector(7 downto 0)
        );
end PixCmdDecodeV461;

architecture Behavioral of PixCmdDecodeV461 is

  constant PixVersionReg : std_logic_vector(15 downto 0) := x"01CD";  -- Version 4.61

  type   PixCmdDecodeState is (PixIdle, PixWrt1, PixRd1, PixRd2, PixRd3);
  signal PixState     : PixCmdDecodeState;
  signal LocalData    : std_logic_vector(31 downto 0);
  signal SerCfg_cs    : std_logic;
  signal ClkCtl_cs    : std_logic;
  signal LedPrt_cs    : std_logic;
  signal SynLnk_cs    : std_logic;
  signal Fetch_cs     : std_logic;
--  signal SerLnk_cs    : std_logic;
--  signal Status_cs    : std_logic;
--  signal ExtDat_cs    : std_logic;
  signal DacDat_cs    : std_logic;
  signal AdcDat_cs    : std_logic;
  signal AdcMode_cs   : std_logic;
  signal firmVers_cs  : std_logic;
  signal pixCtlReg_cs : std_logic;
  signal CsEnable     : std_logic;

  signal CS_4K, CS_200, CS_80, CS_A0, CS_C0, CS_D0, CS_F0 : std_logic;

  signal pixRowReg_cs  : std_logic;
  signal pixColReg_cs  : std_logic;
  signal pixTimeReg_cs : std_logic;
  signal pixChanReg_cs : std_logic;
  signal pixDlyReg_cs  : std_logic;

  signal PixControlReg : std_logic_vector(15 downto 0);
  signal PixRowReg     : std_logic_vector(15 downto 0);
  signal PixColReg     : std_logic_vector(15 downto 0);
  signal PixTimeReg    : std_logic_vector(7 downto 0);
  signal PixChanReg    : std_logic_vector(6 downto 0);
  signal PixDlyReg     : std_logic_vector(7 downto 0);
begin

------------------------------------------------------------------------
-- COMMAND DECODE PROCESS
------------------------------------------------------------------------
  PixCmdDecode : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1') then
        CsEnable <= '0';
        PixState <= PixIdle;
        READ_REQ <= '0';
      else
        case PixState is
          when PixIdle =>
            CsEnable  <= '0';
            LocalData <= CMD_DATA;
            if (CMD_RDY = '1') then
              CMD_ACK <= '1';  -- Clear the command from the interface stack (since we now have a copy)
              if (CMD_READ = '1') then
                READ_WRITE_N <= '1';    -- Read operation required
                PixState     <= PixRd1;
              else
                READ_WRITE_N <= '0';    -- Write operation required
                PixState     <= PixWrt1;
              end if;
            else
              PixState <= PixIdle;
            end if;
          when PixRd1 =>
            CMD_ACK  <= '0';            -- Reset the acknowledge strobe.
            CsEnable <= '1';  -- After 1 clock period to decode, issue a write strobe to the module
            PixState <= PixRd2;
          when PixRd2 =>
            READ_REQ <= '1';  -- Ask for service to the systran board for next clock edge
            PixState <= PixRd3;
          when PixRd3 =>
            if (READ_ACK = '1') then  -- Wait here until the data is accepted.
              READ_REQ <= '0';
              PixState <= PixIdle;
            else
              PixState <= PixRd3;
            end if;
          when PixWrt1 =>
            CMD_ACK  <= '0';            -- Reset the acknowledge strobe.
            CsEnable <= '1';  -- After 1 clock period to decode, issue a write strobe to the module
            PixState <= PixIdle;        -- Go back to look for more commands
          when others =>
            PixState <= PixIdle;
        end case;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- ADDRESS DECODER.
------------------------------------------------------------------------
  CS_4K  <= not (LocalData(31) or LocalData(30) or LocalData(29) or LocalData(28));
  CS_200 <= LocalData(25) and not (LocalData(27) or LocalData(26) or LocalData(24));
  CS_80  <= LocalData(23) and not (LocalData(22) or LocalData(21) or LocalData(20));
  CS_A0  <= LocalData(23) and LocalData(21) and not (LocalData(22) or LocalData(20));
  CS_C0  <= LocalData(23) and LocalData(22) and not (LocalData(21) or LocalData(20));
  CS_D0  <= LocalData(23) and LocalData(22) and LocalData(20) and not LocalData(21);
  CS_F0  <= LocalData(23) and LocalData(22) and LocalData(21) and LocalData(20);

-- 0x0200 => 0x027F   Serial eeprom data area
  SerCfg_cs     <= CsEnable and CS_4K and CS_200 and not LocalData(23);
-- 0x0280   serial eeprom control register
  Fetch_cs      <= CsEnable and CS_4K and CS_200 and CS_80 and not (LocalData(19) or LocalData(18) or LocalData(17) or LocalData(16));
-- 0x0281   System clock phase adjustment register
  ClkCtl_cs     <= CsEnable and CS_4K and CS_200 and CS_80 and not (LocalData(19) or LocalData(18) or LocalData(17)) and LocalData(16);
-- 0x0282   Pixel front panel led function select register
  LedPrt_cs     <= CsEnable and CS_4K and CS_200 and CS_80 and not (LocalData(19) or LocalData(18) or LocalData(16)) and LocalData(17);
-- 0x0283   Syncronization bit register
  SynLnk_cs     <= CsEnable and CS_4K and CS_200 and CS_80 and not (LocalData(19) or LocalData(18)) and LocalData(17) and LocalData(16);
-- 0x0284   High speed serial link register
--    SerLnk_cs  <= CsEnable and CS_4K and CS_200 and CS_80 and not (LocalData(19) or LocalData(17) or LocalData(16)) and LocalData(18);
-- 0x0286   Systran port spare signals
--    ExtDat_cs  <= CsEnable and CS_4K and CS_200 and CS_80 and not LocalData(19) and LocalData(18) and LocalData(17) and not LocalData(16);
-- 0x0288   Debug register control 
--    dbgAddr_cs   <= CsEnable and CS_4K and CS_200 and CS_80 and LocalData(19) and not LocalData(18) and not LocalData(17) and not LocalData(16);
-- 0x02A0 => 0x02A7  DAC output data registers
  DacDat_cs     <= CsEnable and CS_4K and CS_200 and CS_A0 and not LocalData(19);
-- 0x02C0 => 0x02C7  External telemetry ADC input data registers
  AdcDat_cs     <= CsEnable and CS_4K and CS_200 and CS_C0 and not LocalData(19);
-- 0x02C8 => 0x02CF  External telemetry ADC mode register
  AdcMode_cs    <= CsEnable and CS_4K and CS_200 and CS_C0 and LocalData(19);
-- 0x02D0   Simulator Row count register
  pixRowReg_cs  <= CsEnable and CS_4K and CS_200 and CS_D0 and not (LocalData(19) or LocalData(18) or LocalData(17) or LocalData(16));
-- 0x02D1   Simulator Column count register
  pixColReg_cs  <= CsEnable and CS_4K and CS_200 and CS_D0 and LocalData(16) and not (LocalData(19) or LocalData(18) or LocalData(17));
-- 0x02D2   Simulator Pixel time register (units of 100ns)
  pixTimeReg_cs <= CsEnable and CS_4K and CS_200 and CS_D0 and LocalData(17) and not (LocalData(19) or LocalData(18) or LocalData(16));
-- 0x02D3   Simulator Channel count register (number of video channels to simulate)
  pixChanReg_cs <= CsEnable and CS_4K and CS_200 and CS_D0 and LocalData(16) and LocalData(17) and not (LocalData(19) or LocalData(18));
-- 0x02D4   Simulator Initial delay register (units of 1 millisecond)
  pixDlyReg_cs  <= CsEnable and CS_4K and CS_200 and CS_D0 and LocalData(18) and not (LocalData(19) or LocalData(17) or LocalData(16));
-- 0x02FC    Local control register - bit significant
  pixCtlReg_cs  <= CsEnable and CS_4K and CS_200 and CS_F0 and LocalData(19) and LocalData(18) and not (LocalData(17) or LocalData(16));
-- 0x02FD    Local status register
--    Status_cs  <= CsEnable and CS_4K and CS_200 and CS_F0 and LocalData(19) and LocalData(18) and LocalData(16) and not LocalData(17);
-- 0x02FF    Local firmware readback register
  firmVers_cs   <= CsEnable and CS_4K and CS_200 and CS_F0 and LocalData(19) and LocalData(18) and LocalData(17) and LocalData(16);
------------------------------------------------------------------------
-- DATA, ADDRESS, AND CHIP SELECT DEMUX.
------------------------------------------------------------------------
  DATA_OUT      <= LocalData(15 downto 0);
  ADDRESS       <= LocalData(22 downto 16);
  CFG_CMD_STB   <= SerCfg_cs;
  ADC_CMD_STB   <= AdcDat_cs;
  MOD_CMD_STB   <= AdcMode_cs;
  DAC_CMD_STB   <= DacDat_cs;
--    EXD_CMD_STB <= ExtDat_cs;
  SYN_CMD_STB   <= SynLnk_cs;
  CLK_CMD_STB   <= ClkCtl_cs;
  FCH_CMD_STB   <= Fetch_cs;
  LED_CMD_STB   <= LedPrt_cs;

  READ_SYNC <= '1';                     -- Until further notice !

------------------------------------------------------------------------
-- Control and simulation registers + trigger functions
------------------------------------------------------------------------
  PixControlRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (READ_WRITE_N = '0' and pixCtlReg_cs = '1') then
        PixControlReg <= LocalData(15 downto 0);
      end if;
    end if;
  end process;

  DBL_WRD_XFER <= PixControlReg(0);
  SIM_ENABLE   <= PixControlReg(15);

  PixRowRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (READ_WRITE_N = '0' and pixRowReg_cs = '1') then
        PixRowReg <= LocalData(15 downto 0);
      end if;
    end if;
  end process;
  SIM_PIXROWS <= PixRowReg;

  PixColRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (READ_WRITE_N = '0' and pixColReg_cs = '1') then
        PixColReg <= LocalData(15 downto 0);
      end if;
    end if;
  end process;
  SIM_PIXCOLS <= PixColReg;

  PixTimeRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (READ_WRITE_N = '0' and pixTimeReg_cs = '1') then
        PixTimeReg <= LocalData(7 downto 0);
      end if;
    end if;
  end process;
  SIM_PIXTIME <= PixTimeReg;

  PixChanRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (READ_WRITE_N = '0' and pixChanReg_cs = '1') then
        PixChanReg <= LocalData(7 downto 1);
      end if;
    end if;
  end process;
  SIM_CHANNELS <= PixChanReg;

  PixDlyRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (READ_WRITE_N = '0' and pixDlyReg_cs = '1') then
        PixDlyReg <= LocalData(7 downto 0);
      end if;
    end if;
  end process;
  SIM_DELAY <= PixDlyReg;

  REG_DATA <= PixControlReg            when (pixCtlReg_cs = '1' and READ_WRITE_N = '1')  else (others => 'Z');
  REG_DATA <= PixRowReg                when (pixRowReg_cs = '1' and READ_WRITE_N = '1')  else (others => 'Z');
  REG_DATA <= PixColReg                when (pixColReg_cs = '1' and READ_WRITE_N = '1')  else (others => 'Z');
  REG_DATA <= x"00" & PixTimeReg       when (pixTimeReg_cs = '1' and READ_WRITE_N = '1') else (others => 'Z');
  REG_DATA <= x"00" & '1' & PixChanReg when (pixChanReg_cs = '1' and READ_WRITE_N = '1') else (others => 'Z');
  REG_DATA <= x"00" & PixDlyReg        when (pixDlyReg_cs = '1' and READ_WRITE_N = '1')  else (others => 'Z');
  REG_DATA <= PixVersionReg            when (firmVers_cs = '1' and READ_WRITE_N = '1')   else (others => 'Z');

end Behavioral;
