------------------------------------------------------------------------
-- Copyright Notice:                                                    
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA U25
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   S-LINK INTERFACE
-- VERSION      :   1.0 
------------------------------------------------------------------------
-- FILE NAME    :   SLINKIFC.VHD
-- AUTHOR       :   MIQUEL BARCELO 
-- DATE         :   ??/??/????
-- COMPANY      :   IFE
-- PLATFORM     :   Xilinx ISE 7.1
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES THE INTERFACE BETWEEN THE PIXEL FPGA AND THE
-- S-LINK UPLINK AND RETURN LINK PORT.  
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
--
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE        AUTHOR  VERSION  MODIFICATIONS
-- ??/??/????  MIQUEL  1.0      ORIGINAL VERSION
-- 4 Dec 2008  JTO              replace UCLK output with ODDR flip flop
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK      : SYSTEM CLOCK
-- RESET_IN    : ASYNCHRONOUS MODULE RESET
-- DATA_IN     : 32 BIT DATA TO BE SENT VIA UPLINK TO PAN COMPUTER
-- DATA_WRT    : STROBE TO WRITE DATA_IN INTO MODULE ON RISING EDGE OF SYSCLK
-- SYNC_IN     : STROBE SENSED DURING DATA_WRT TO IDENTIFY DATA TYPE - LOW = PIXEL DATA, HI = MESSAGE DATA
-- DATA_BUSY   : SIGNAL TO INHIBIT DATA_WRT STROBE. LINK DOWN OR OVER CAPACITY
--
-- DATA_OUT    : 32 BIT DATA FROM THE PAN COMPUTER VIA THE RETURN LINK
-- DATA_RDY    : STROBE TO INDICATE DATA AVAILABLE ON DATA_OUT ON RISING EDGE OF SYSCLK
-- DATA_READ   : STROBE TO INDICATE THAT DATA_OUT DATA HAS BEEN LATCHED EXTERNALLY
-- RESET_OUT   : LATCHED SIGNAL TO IDENTIFY A RESET CONDITION SENT BY THE PAN COMPUTER. RESET BY RESET_IN
--
-- UCLK        : HOLA LSC MASTER CLOCK
-- UD          : HOLA LSC UPLINK DATA PORT
-- UWEN_N      : HOLA LSC WRITE STROBE FOR UD DATA
-- UCTRL_N     : HOLA LSC CONTROL WORD STROBE
-- LRL         : HOLA LSC RETURN DATA PORT
-- LFF_N       : HOLA LSC LINK FULL FLAG
-- UTEST_N     : HOLA LSC LINK LOOP BACK SIGNAL
-- LDOWN_N     : HOLA LSC LINK STATUS SIGNAL
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library unisim;
use UNISIM.vcomponents.all;

entity SLinkIfc is
-- generic (
-- page_size : integer := 4096;
-- wait_cycles : integer := 256
-- );
  port (DATA_IN   : in  std_logic_vector (31 downto 0);
        DATA_READ : in  std_logic;
        DATA_WRT  : in  std_logic;
        LDOWN_N   : in  std_logic;
        LFF_N     : in  std_logic;
        LRL       : in  std_logic_vector (3 downto 0);
        RESET_IN  : in  std_logic;
        SYNC_IN   : in  std_logic;
        SYSCLK    : in  std_logic;
        DATA_BUSY : out std_logic;
        DATA_OUT  : out std_logic_vector (31 downto 0);
        DATA_RDY  : out std_logic;
        RESET_OUT : out std_logic;
        UCLK      : out std_logic;
        UCTRL_N   : out std_logic;
        UD        : out std_logic_vector (31 downto 0);
        UTEST_N   : out std_logic;
        URESET_N  : out std_logic;
        UWEN_N    : out std_logic);
end SLinkIfc;

architecture BEHAVIORAL of SLinkIfc is
  component lrl_decoder
    port (
      areset    : in  std_logic;
      sysclk    : in  std_logic;
      lrl       : in  std_logic_vector (3 downto 0);
      reset_out : out std_logic;
      data      : out std_logic_vector (31 downto 0);
      wrden     : out std_logic;
      wrcen     : out std_logic
      );
  end component;

  component slink_fifo
    generic (
      width : integer := 32;
      depth : integer := 6
      );
    port (
      clk    : in  std_logic;
      areset : in  std_logic;
      din    : in  std_logic_vector (width-1 downto 0);
      wr     : in  std_logic;
      dout   : out std_logic_vector (width-1 downto 0);
      rd     : in  std_logic;
      empty  : out std_logic;
      full   : out std_logic
      );
  end component;

  -- Signals for loop mode muxs

  signal int_mux_datain  : std_logic_vector(31 downto 0);
  signal int_mux_syncin  : std_logic;
  signal int_mux_datawr  : std_logic;
  signal int_mux_busy    : std_logic;
  signal int_mux_dataout : std_logic_vector(31 downto 0);
  signal int_mux_datard  : std_logic;


  signal int_data     : std_logic_vector(31 downto 0);
  signal int_wren     : std_logic;
  signal int_empty    : std_logic;
  signal int_fw_empty : std_logic;
  signal int_fw_read  : std_logic;
  signal int_fw_sync  : std_logic;
  signal int_fw_data  : std_logic_vector(31 downto 0);

  signal int_page_flag, int_page_ena, int_page_rst : std_logic;
  signal int_wait_flag, int_wait_ena, int_wait_rst : std_logic;

  signal int_wrcen, areset          : std_logic;
  signal int_page_ctr, int_wait_ctr : std_logic_vector(15 downto 0);
  signal reg_num_page, reg_num_wait : std_logic_vector(15 downto 0);

  signal reg_loop_mode                        : std_logic_vector(1 downto 0);
  signal reg_num_page_ena, reg_num_wait_ena   : std_logic;
  signal reg_loop_mode_ena                    : std_logic;
  signal rLFF_N, rrLFF_N, rLDOWN_N, rrLDOWN_N : std_logic;
  signal rUWEN_N, rUCTRL_N, rURESET_N         : std_logic;
  signal rUD                                  : std_logic_vector(31 downto 0);
  signal mux_fw_data                          : std_logic_vector(31 downto 0);
--  signal link_ready                                : std_logic;

  type   sm_type is (reset, idle, ctrl_scw, ctrl_send, ctrl_ecw, data_scw, data_send, data_ecw);
  signal state, next_state : sm_type;


begin

  decoder : lrl_decoder
    port map (
      areset    => areset,
      sysclk    => SYSCLK,
      lrl       => LRL,
      data      => int_data,
      wrden     => int_wren,
      wrcen     => int_wrcen,
      reset_out => RESET_OUT
      );

-- DATA_OUT <= std_logic_vector(to_unsigned(max_page_ctr, 16)) & "000"& int_wrcen & int_data(11 downto 0);
-- DATA_OUT <= reg_num_page & reg_num_wait;
  databuffer : slink_fifo
    generic map (
      width => 32,
      depth => 6)
    port map (
      areset => areset,
      clk    => SYSCLK,
      din    => int_data,
      wr     => int_wren,
      dout   => int_mux_dataout,
-- full => RESET_OUT,
      rd     => int_mux_datard,
      empty  => int_empty
      );

  fw_buffer : slink_fifo
    generic map (
      width => 32,
      depth => 6)
    port map (
      areset => areset,
      clk    => SYSCLK,
      din    => int_mux_datain,
      wr     => int_mux_datawr,
      dout   => int_fw_data,
      full   => int_mux_busy,
      rd     => int_fw_read,
      empty  => int_fw_empty
      );

  fw_sync_buffer : slink_fifo
    generic map (
      width => 1,
      depth => 6)
    port map (
      areset  => areset,
      clk     => SYSCLK,
      din(0)  => int_mux_syncin,
      wr      => int_mux_datawr,
      dout(0) => int_fw_sync,
      rd      => int_fw_read
      );

  -- Loop mode: Return FIFO
  DATA_OUT <= X"1BACAFEA" when reg_loop_mode(0) = '1' else
              int_mux_dataout;
  DATA_RDY <= '0' when reg_loop_mode(0) = '1' else
              not int_empty;
  int_mux_datard <= not int_mux_busy when reg_loop_mode(0) = '1' else
                    DATA_READ;
  -- Loop mode: Forward FIFO
  DATA_BUSY <= '1' when reg_loop_mode(0) = '1' else
               int_mux_busy;
  int_mux_datawr <= not int_empty when reg_loop_mode (0) = '1' else
                    DATA_WRT;
  int_mux_datain <= int_mux_dataout when reg_loop_mode(0) = '1' else
                    DATA_IN;
  int_mux_syncin <= reg_loop_mode(1) when reg_loop_mode(0) = '1' else
                    SYNC_IN;
  
  reg_inputs : process (SYSCLK, areset)
  begin
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' then
        rLFF_N    <= '0';
        rrLFF_N   <= '0';
        rLDOWN_N  <= '0';
        rrLDOWN_N <= '0';
      else
        rLFF_N    <= LFF_N;
        rrLFF_N   <= rLFF_N;
        rLDOWN_N  <= LDOWN_N;
        rrLDOWN_N <= rLDOWN_N;
      end if;
    end if;
  end process reg_inputs;

  register_num_page : process (SYSCLK, areset)
  begin
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' then reg_num_page              <= X"0FFE";
      elsif reg_num_page_ena = '1' then reg_num_page <= int_data(15 downto 0);
      else reg_num_page                              <= reg_num_page;
      end if;
    end if;
  end process register_num_page;

  register_num_wait : process (SYSCLK, areset)
  begin
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' then reg_num_wait              <= X"3FFF";
      elsif reg_num_wait_ena = '1' then reg_num_wait <= int_data(15 downto 0);
      else reg_num_wait                              <= reg_num_wait;
      end if;
    end if;
  end process register_num_wait;

  register_loop_mode : process (SYSCLK, areset)
  begin
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' then
        reg_loop_mode <= "00";
      elsif reg_loop_mode_ena = '1' then
        reg_loop_mode <= int_data(1 downto 0);
      else
        reg_loop_mode <= reg_loop_mode;
      end if;
    end if;
  end process register_loop_mode;

  counter_page : process (SYSCLK, areset)
  begin
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' or int_page_rst = '1' then
        int_page_ctr <= (others => '0');
      elsif int_page_ena = '1' then
        int_page_ctr <= std_logic_vector(unsigned(int_page_ctr) + 1);
      else
        int_page_ctr <= int_page_ctr;
      end if;
    end if;
  end process counter_page;

  counter_wait : process (SYSCLK, areset)
  begin
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' or int_wait_rst = '1' then
        int_wait_ctr <= (others => '0');
      elsif int_wait_ena = '1' then
        int_wait_ctr <= std_logic_vector(unsigned(int_wait_ctr) + 1);
      else
        int_wait_ctr <= int_wait_ctr;
      end if;
    end if;
  end process counter_wait;

  ---------------------------------------------------------------------------
  --  State Machine
  ---------------------------------------------------------------------------

  next_state <=
    reset     when areset = '0'                                                     else
    reset     when state = reset and rrLDOWN_N = '0'                                else
    idle      when state = reset                                                    else
    state     when rrLFF_N = '0'                                                    else
    idle      when state = idle and int_fw_empty = '1'                              else
    ctrl_scw  when state = idle and int_fw_sync = '1'                               else
    data_scw  when state = idle and int_fw_sync = '0'                               else
    ctrl_send when state = ctrl_scw                                                 else
    ctrl_ecw  when state = ctrl_send and int_fw_empty = '1'                         else
    ctrl_ecw  when state = ctrl_send and int_fw_sync = '0'                          else
    ctrl_send when state = ctrl_send                                                else
    ctrl_ecw  when state = ctrl_send                                                else
    data_send when state = data_scw                                                 else
    data_ecw  when state = data_send and int_fw_sync = '1'                          else
    data_ecw  when state = data_send and int_wait_flag = '1'                        else
    data_ecw  when state = data_send and int_page_flag = '1' and int_fw_empty = '0' else
    data_send when state = data_send                                                else
    idle;

  state_machine : process (sysclk, areset)
  begin  -- process state_machine
    if sysclk'event and sysclk = '1' then  -- rising clock edge
      if areset = '0' then state <= idle;
      else state                 <= next_state;
      end if;
    end if;
  end process state_machine;

  int_fw_read <=
    '0' when rrLFF_N = '0'                                                   else
    '0' when int_fw_empty = '1'                                              else
    '1' when state = ctrl_send and int_fw_sync = '1'                         else
    '1' when state = data_send and int_fw_sync = '0' and int_page_flag = '0' else
    '0';
  reg_loop_mode_ena <=
    '1' when int_wrcen = '1' and int_data(31 downto 28) = "0010" else
    '0';
  reg_num_page_ena <=
    '1' when int_wrcen = '1' and int_data(31 downto 28) = "0001" else
    '0';
  reg_num_wait_ena <=
    '1' when int_wrcen = '1' and int_data(31 downto 28) = "0000" else
    '0';
  int_wait_flag <=
    '1' when int_wait_ctr = reg_num_wait else
    '0';
  int_page_flag <=
    '1' when int_page_ctr = reg_num_page else
    '0';
  int_page_rst <=
    '0' when state = data_send else
    '1';
  int_wait_rst <=
    '0' when state = data_send and int_fw_empty = '1' else
    '1';
  int_page_ena <=
    '1' when rrLFF_N = '1' and int_fw_empty = '0' and state = data_send else
    '0';
  int_wait_ena <=
    '1' when rrLFF_N = '1' and int_fw_empty = '1' and state = data_send else
    '0';

  with state select
    mux_fw_data <=
    X"B0F0C000" when ctrl_scw,
    int_fw_data when ctrl_send,
    X"E0F0C000" when ctrl_ecw,
    X"B0F0D000" when data_scw,
    int_fw_data when data_send,
    X"E0F0D000" when data_ecw,
    X"0BADDA7A" when others;

  rUD <= X"0BADDA7A" when rrLFF_N = '0' else
         mux_fw_data;

  rUCTRL_N <=
    '0' when state = ctrl_scw else
    '0' when state = ctrl_ecw else
    '0' when state = data_scw else
    '0' when state = data_ecw else
    '1';
  rUWEN_N <=
    '1' when rrLFF_N = '0'                           else
    '0' when state = ctrl_scw                        else
    '0' when state = ctrl_send and int_fw_read = '1' else
    '0' when state = ctrl_ecw                        else
    '0' when state = data_scw                        else
    '0' when state = data_send and int_fw_read = '1' else
    '0' when state = data_ecw                        else
    '1';
  rURESET_N <=
    '0' when state = reset else
    '1';
-- URESET_N <= areset;

  registers_output : process (SYSCLK, areset)
  begin  -- process registers_output
    if SYSCLK'event and SYSCLK = '1' then
      if areset = '0' then
        UD       <= (others => '0');
        UCTRL_N  <= '1';
        UWEN_N   <= '1';
        URESET_N <= '0';
-- URESET_N <= '1';
      else
        UD       <= rUD;
        UCTRL_N  <= rUCTRL_N;
        UWEN_N   <= rUWEN_N;
        URESET_N <= rURESET_N;
-- URESET_N <= '1';
      end if;
    end if;
  end process registers_output;

-- URESET_N <= '1';
  areset  <= not RESET_IN;
  UTEST_N <= '1';
-- DATA_RDY <= not int_empty;

-- UCLK <= not SYSCLK;

  -- use a DDR output register, which works like this

  -- OPPOSITE_EDGE mode - The data transmit interface uses the classic DDR methodology.
  -- Given a data and clock at pin D1-2 and C respectively, D1 is sampled at every positive
  -- edge of clock C, and D2 is sampled at every negative edge of clock C. 
  -- Q changes every clock edge.

  ODDR_inst : ODDR
    generic map(
      DDR_CLK_EDGE => "OPPOSITE_EDGE",  -- "OPPOSITE_EDGE" or "SAME_EDGE"
      INIT         => '0',  -- Initial value for Q port (1 or 0)
      SRTYPE       => "SYNC"            -- Reset Type ("ASYNC" or "SYNC")
      )
    port map(
      C  => sysclk,
      CE => '1',
      D1 => '0',
      D2 => '1',
      R  => '0',
      S  => '0',
      Q  => UCLK
      );

end BEHAVIORAL;
