------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA (U20)
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   SERIAL EEPROM READ / WRITE LOGIC
-- VERSION      :   4.2 
------------------------------------------------------------------------
-- FILE NAME    :   SERCONFIGIFC.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/23/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   THIS MODULE IMPLEMENTS A SIMPLE 12C INTERFACE TO ENABLE
-- IT TO READ AND WRITE TO A SERIAL EEPROM. FOR MONSOON, THIS EEPROM STORES
-- SPECIFIC SYSTEM ATTRIBUTES THAT THE PAN USES TO BOOT THE SYSTEM 
-- CONFIGURATION
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- THE LOGIC IS INITIALLY TRIGGERED BY THE RESET SIGNAL HOWEVER, PROVISION
-- IS MADE TO ALLOW AN EXTERNAL COMMAND TO INITIATE A READ SEQUENCE. THE
-- EEPROM DATA IS STORED IN A SHADOW RAM THAT COPIES THE CONTENTS OF THE 
-- EEPROM WHEN TRIGGERED. FOR A 24C02 DEVICE, THIS CORRESPONDS TO 128X16 BIT
-- WORDS. DURING THE RESET TRIGGER THE LOGIC SEARCHES FOR AN EEPROM DEVICE
-- THAT ASSERTS THE ACKNOWLEDGE BIT. THE SEARCH IS CONDUCTED BY FIRST LOOKING
-- FOR A DEVICE WITH ADDRESS 000 (E0,E1,E2 = '0'). SINCE THE MCB RESIDENT DEVICE
-- HAS AN ADDRESS OF 1, ANY DEVICE ON A TRANSITION BOARD WITH ADDRESS 0 WILL BE
-- READ FIRST. BY ASSERTING THE FETCH SIGNAL, ANY ADDRESS CAN BE ACCESSED VIA THE
-- LEAST SIGNIFICANT 3 BITS OF THE DATA_ADDR VECTOR. USING THE FETCH COMMAND WILL
-- OVER WRITE ANY OTHER DATA IN TEH SHADOW RAM. ONLY 127 LOCATIONS OF THE SHADOW 
-- RAM CAN BE ACCESSED. THE LAST ADDRESS (0x7F) CONTAINS THE MODULES STATUS WORD.
-- STATUS WORD SIGNIFICANCE IS:
-- BIT  0   LSB OF PHYSICAL DEVICE
-- BIT  1          PHYSICAL DEVICE
-- BIT  2   MSB OF PHYSICAL DEVICE
-- BIT  3   ALWAYS ZERO
-- BIT  4   LSB OF SHADOW RAM ADDR
-- BIT  5          SHADOW RAM ADDR   
-- BIT  6          SHADOW RAM ADDR
-- BIT  7          SHADOW RAM ADDR
-- BIT  8          SHADOW RAM ADDR
-- BIT  9          SHADOW RAM ADDR
-- BIT 10   MSB OF SHADOW RAM ADDR
-- BIT 11   ALWAYS ZERO
-- BIT 12   READ PROCESS ENABLE FLAG 
-- BIT 13   CLOCK PROCESS ENABLE FLAG
-- BIT 14   RAM DATA VALID
-- BIT 15   EEPROM READ IN PROGRESS
--
-- EVEN THE BEST LAID PLANS ...
-- NOTE THAT SOME 24C02 DEVICES DO NOT USE THE THREE EXTERNAL ADDRESS BITS.
-- THESE DEVICES WILL ANSWER BACK TO ANY SELECT ADDRESS WRITEN TO IT ... HENCE,
-- WITH THESE DEVICES IT IS NOT POSSIBLE TO HAVE TWO EEPROM DEVICES ON THE SAME BUS ! 
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE        AUTHOR   VERSION  MODIFICATIONS
-- 04/14/2004   PCM    4.0  INITIAL VERSION
-- 05/05/2004   PCM    4.1  REWRITTEN TO INCLUDE WRITE PROCESS.
-- 05/23/2004   PCM    4.2  REWRITTEN TO MAKE IT WORK !
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK   : SYSTEM CLOCK
-- RESET    : RESET THE SHADOW RAM SIGNALS AND INITIATE A SEARCH FOR EEPROMs
-- FETCH    : TRIGGERS A READ OF AN EEPROM WHOS ADDRESS IS THE SAME AS THE DATA_ADDR VALUE
-- SRCFG_CS : REQUEST TO READ THE SHADOW RAM WITH ADDRESS IN DATA_ADDR
-- DATA_OUT : DATA FROM SHADOW RAM TO SYSTRAN VIA MULTIPLEXOR
-- DATA_ADDR: ADDRESS OF DATA REQUIRED OR ADDRESS OF EEPROM DURING A FETCH CMD
-- DATA_RDY : SIGNAL TO THE SYSTRAN MULTIPLEXOR FOR DATA XFER TO PAN
-- READ     : ACKNOWLEDGE FROM SYSTRAN DATA MULTIPLEXOR
-- SCL      : I2C SERIAL CLOCK LINE
-- SDA_IN   : I2C SERIAL DATA SIGNAL FROM DEVICE
-- SDA_OUT  : I2C SERIAL DATA TO DEVICE VIA TRISTATE BUFFER
-- SDA_ENAB : SDA ENABLE FOR SDA_OUT WHEN TRUE  
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity SerConfigIfcV42 is
  port (SYSCLK     : in    std_logic;
         RESET     : in    std_logic;
         READ      : in    std_logic;
         FETCH_CS  : in    std_logic;
         SRCFG_CS  : in    std_logic;
         WRITE_EN  : in    std_logic;
         DATA_IN   : in    std_logic_vector(15 downto 0);
         DATA_OUT  : inout std_logic_vector(15 downto 0);
         DATA_ADDR : in    std_logic_vector(6 downto 0);
         SCL       : out   std_logic;
         SDA_IN    : in    std_logic;
         SDA_OUT   : out   std_logic;
         SDA_ENAB  : out   std_logic;
         PING      : out   std_logic;
         PONG      : out   std_logic
         );
end SerConfigIfcV42;

architecture Behavioral of SerConfigIfcV42 is
-- Constants used in the logic here
  constant MAX_SRCH_ADDR                   : std_logic_vector(2 downto 0)  := "100";  -- Maximum physical address that read routine will search for after reset.
  constant CLK_PERIOD                      : std_logic_vector(5 downto 0)  := "101000";  -- Serial clock phase period in terms of sysclk. Nominally 40 (1000 ns).
  constant RD_STROBE                       : std_logic_vector(5 downto 0)  := "010100";  -- When device data is readable.
  constant TYPE_IDENT                      : std_logic_vector(3 downto 0)  := "1010";  -- Fixed device identity nibble.
  constant BITS_PER_BYTE                   : std_logic_vector(3 downto 0)  := "1000";  -- Number of bits in a byte ... Really !
  constant MAX_RAM_ADDR                    : std_logic_vector(6 downto 0)  := "1111110";  -- Maximum ram address to shadow from EEPROM.
  constant MAX_WRT_RETRY                   : std_logic_vector(7 downto 0)  := "00001111";  -- Number of retries to get an acknowledge when writing to eeprom (Should never need more than 5).
  constant RETRY_PERIOD                    : std_logic_vector(15 downto 0) := x"9C40";  -- period of time between retries = 1ms.
-- Shadow ram store signals
  type     SerCfgStoreType is array (127 downto 0) of std_logic_vector (15 downto 0);
  signal   SerCfgStore                     : SerCfgStoreType;  -- Actual shadow memory for serial EEPROM.
  signal   SerCfgRamAddr                   : std_logic_vector(6 downto 0);
  signal   SerCfgStoreData                 : std_logic_vector(15 downto 0);
  signal   SerCfgDataSrc                   : std_logic_vector(15 downto 0);
  signal   SerCfgStoreWrt                  : std_logic;
  signal   SerCfgWrtStb                    : std_logic;
  signal   loc_data                        : std_logic_vector(15 downto 0);
  signal   Status_read                     : std_logic;
  signal   RdEnable                        : std_logic;  -- Read decode from PAN request
  signal   DevPhyAddr                      : std_logic_vector(2 downto 0);  -- Stores the device physical address during a ext triggered scan.
  signal   SerCfgDev                       : std_logic_vector(2 downto 0);  -- Current device address in use.
  signal   SerCfgWrtData                   : std_logic;  -- Enabled write strobe to write data to shadow ram from pan.
-- SerCfgSearch process signals
  signal   SerCfgRd                        : std_logic;  -- Actual trigger to start read process.
  signal   RstLockOut                      : std_logic;  -- Locks out the IC2 read routine in case reset signal is permanant. 
  signal   loc_active                      : std_logic;  -- Used to cycle through actions in the search.
  signal   SrchAddr                        : std_logic_vector(2 downto 0);  -- Physical EEPROM address used during search routine
  signal   SrchTrigRdRom                   : std_logic;  -- Triggers a read of the ROM in search mode.
-- Write eeprom FSM signals
  type     CfgWrtStateType is (CfgWrtIdle, CfgWrtWordLoop, CfgWrtSendAddr, CfgWrtDelay, CfgWrtAddrAck, CfgWrtDataByte, CfgWrtDataWait, CfgWrtSendStop);
  signal   CfgWrtState                     : CfgWrtStateType;
  signal   SerCfgWrt                       : std_logic;
  signal   WrtRomActive                    : std_logic;  -- Indicates the write eeprom FSM is active.
  signal   WrtLoadData                     : std_logic_vector (7 downto 0);
  signal   WrtLoadFlag                     : std_logic;
  signal   WrtClkEnable                    : std_logic;
  signal   CfgWrtAddr                      : std_logic_vector(6 downto 0);
  signal   WrtHighByte                     : std_logic;
  signal   WrtStartReq                     : std_logic;  -- Trigger a start + select sequence
  signal   WrtStopReq                      : std_logic;
  signal   WrtDlyCntr                      : std_logic_vector(15 downto 0);  -- Used to time out retries on select.
  signal   WrtRetryCount                   : std_logic_vector(7 downto 0);
-- Read eeprom FSM signals
  type     CfgRdStateType is(CfgRdIdle, CfgRdSendAddr, CfgRdAddrAck, CfgRdReStart, CfgRdRdDat1, CfgRdRdDat2, CfgRdSendStop);
  signal   CfgRdState                      : CfgRdStateType;
  signal   CfgRdAddr                       : std_logic_vector(6 downto 0);
  signal   CfgReadStatus                   : std_logic_vector(15 downto 0);
  signal   RdRomActive                     : std_logic;
  signal   RdRomSuccess                    : std_logic;
  signal   RdLoadFlag                      : std_logic;
  signal   RdClkEnable                     : std_logic;
  signal   RdStartReq                      : std_logic;
  signal   RdStopReq                       : std_logic;
  signal   AckFlag                         : std_logic;
  signal   RdSndStartWriteFlag             : std_logic;
  signal   FirstFlag                       : std_logic;
  signal   RdHighByte                      : std_logic;
-- Start sequence process signals
  type     StrtStateType is (StrtIdle, StrtSync, StrtStart, StrtSendSlct, StrtAcknowledge, StrtEndFrame);
  signal   StrtState                       : StrtStateType;
  signal   SendStartReq                    : std_logic;  -- Global request for a start sequence
  signal   StartBusy                       : std_logic;  -- signal that stop sequence is in progress
  signal   StrtLoadFlag                    : std_logic;  -- Request from stop sequence to load data to sda line
  signal   StrtClkEnable                   : std_logic;  -- Request from stop generator for use of clock generator
  signal   StrtLoadData                    : std_logic_vector (7 downto 0);  -- Data for stop generator
  signal   WriteFlag                       : std_logic;
-- Stop sequence process signals
  type     SndStopStateType is (SndStopIdle, SndStopStart, SndStopStop);
  signal   SndStopState                    : SndStopStateType;
  signal   SendStopReq                     : std_logic;  -- Global request for a stop sequence
  signal   StopBusy                        : std_logic;  -- signal that stop sequence is in progress
  signal   StpLoadFlag                     : std_logic;  -- Request from stop sequence to load data to sda line
  signal   StpClkEnable                    : std_logic;  -- Request from stop generator for use of clock generator
  signal   StpLoadData                     : std_logic_vector (7 downto 0);  -- Data for stop generator
-- SerCfgClkGen process signals
  signal   ClkEnableFlag                   : std_logic;  -- Enables the clock generator to run.
  signal   ClkCntr                         : std_logic_vector(5 downto 0);  -- Counts sysclk periods to derive the I2C serial clock.
  signal   SerCfgClk                       : std_logic_vector(1 downto 0);  -- Generates three clock phases .. 00, 01, 10 Bit 0 is inverted and used as the SCL clock. 
  signal   RdStrobe                        : std_logic;  -- Ocurres in middle of clock phase 01 when SDA data should be read.
  signal   ClkPhase0, ClkPhase1, ClkPhase2 : std_logic;  -- Decoded clock phase information.
-- SerCfgWrtData process signals
  type     WrtDataStateType is (WrtDataIdle, WrtDataBusy, WrtDataAcknowledge, WrtDataEndFrame);
  signal   WrtDataState                    : WrtDataStateType;
  signal   WrtSdaEnable                    : std_logic;
  signal   WriteBusy                       : std_logic;
  signal   LoadData                        : std_logic_vector(7 downto 0);
  signal   LoadFlag                        : std_logic;  -- Global load flag for write process
  signal   SerWrtData                      : std_logic_vector(7 downto 0);  -- Parallel to serial conversion register.
  signal   DataArm                         : std_logic;  -- One shot toggle to slow things down
  signal   BitCntr                         : std_logic_vector(3 downto 0);  -- Serial SDA bit counter
  signal   SdaEnable                       : std_logic;  -- Enables control over the sda data line
  signal   SdaAck                          : std_logic;  -- Result of acknowledge bit time from slave device.
-- SerCfgRdData process signals
  signal   SerRdData                       : std_logic_vector(7 downto 0);  -- Actual serial to parallel conversion register.
  signal   RdByteValue                     : std_logic_vector(7 downto 0);  -- Latched value of byte read in by process. Valid with RdDataRdy.
  signal   RdBitCntr                       : std_logic_vector(3 downto 0);  -- Counts the number of bits read in.
  signal   RdEnableFlag                    : std_logic;  -- Lets the read process run and collect data
  signal   RdDataRdy                       : std_logic;  -- Flag to indicate that data is ready to be played with
  signal   ShiftFlag                       : std_logic;  -- Used to control serial > parallel shift register.
  signal   AckSdaEnable                    : std_logic;  -- Enables the tristate buffer for SDA master ACKNOWLEDGE signal
  signal   RdAckActive                     : std_logic;  -- Used during the read acknowledge cycle.
begin

-- Debug --
  PING <= RdDataRdy;
  PONG <= LoadFlag;

------------------------------------------------------------------------
-- Shadow RAM address multiplexor and function address decoder.
------------------------------------------------------------------------
  SerCfgRamAddr <= CfgRdAddr when (RdRomActive = '1') else
                   CfgWrtAddr when (WrtRomActive = '1') else
                   DATA_ADDR;
  SerCfgDataSrc <= SerCfgStoreData when (RdRomActive = '1') else DATA_IN;
  SerCfgWrtStb  <= SerCfgStoreWrt  when (RdRomActive = '1') else SerCfgWrtData;
  Status_read   <= DATA_ADDR(0) and DATA_ADDR(1) and DATA_ADDR(2) and DATA_ADDR(3) and DATA_ADDR(4) and DATA_ADDR(5) and DATA_ADDR(6);
  SerCfgWrtData <= (SRCFG_CS and WRITE_EN and not READ);

------------------------------------------------------------------------
-- Shadow RAM read and write processes.
------------------------------------------------------------------------
  SerCfgWrite : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (SerCfgWrtStb = '1') then
        SerCfgStore(conv_integer(SerCfgRamAddr)) <= SerCfgDataSrc;
      end if;
    end if;
  end process;

  loc_data <= SerCfgStore(conv_integer(SerCfgRamAddr));

------------------------------------------------------------------------
-- Shadow RAM data multiplexor.
------------------------------------------------------------------------
  RdEnable <= SRCFG_CS and READ;
  DATA_OUT <= CfgReadStatus               when (Status_read = '1' and RdEnable = '1')                       else (others => 'Z');
  DATA_OUT <= x"FFFF"                     when (RdRomActive = '1' and RdEnable = '1' and Status_read = '0') else (others => 'Z');
  DATA_OUT <= loc_data                    when (RdRomActive = '0' and RdEnable = '1' and Status_read = '0') else (others => 'Z');
  DATA_OUT <= (x"000" & '0' & DevPhyAddr) when (FETCH_CS = '1' and READ = '1')                              else (others => 'Z');

  CfgReadStatus <= RdRomActive & RdRomSuccess & ClkEnableFlag & RdEnableFlag & '0' & CfgRdAddr & '0' & SerCfgDev;

------------------------------------------------------------------------
-- Initiate a search for devices on reset.
------------------------------------------------------------------------
  SerCfgSearch : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1') then
        if (RstLockOut = '0') then
          RstLockOut    <= '1';  -- Lock ourselves out from passing this way again until another reset.
          SrchAddr      <= (others => '0');  -- Set the search start physical address 0
          SrchTrigRdRom <= '1';         -- Trigger the process
          loc_active    <= '0';  -- Lock out any other action until ReadRom starts
        elsif (RdRomActive = '1') then
          SrchTrigRdRom <= '0';  -- The read process is active so kill the trigger.
          loc_active    <= '1';
        elsif (RdRomSuccess = '0' and loc_active = '1') then
          if (SrchAddr < MAX_SRCH_ADDR) then
            SrchAddr      <= SrchAddr + 1;   -- Increment the device address
            SrchTrigRdRom <= '1';       -- and trigger another search.
            loc_active    <= '0';  -- Lock out this condition until ReadRom starts again.
          end if;
        end if;
      else
        RstLockOut <= '0';
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- PAN Initiated read and write process triggers and
-- EEPROM physical device multiplexor.
------------------------------------------------------------------------
  SerCfgTrigRd : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RdRomActive = '0' and WrtRomActive = '0') then  -- We're currently in idle state
        if (SrchTrigRdRom = '1') then   -- We have work to do
          SerCfgDev <= SrchAddr;
          SerCfgRd  <= '1';
        elsif (FETCH_CS = '1' and READ = '0') then
          SerCfgDev  <= DATA_IN(2 downto 0);  -- Catch the relevent physical device address.
          DevPhyAddr <= DATA_IN(2 downto 0);  -- and stash it so PAN read back can be done.
          SerCfgRd   <= '1';  -- And trigger the readout of the EEPROM.
        elsif (SerCfgWrtData = '1' and Status_read = '1') then
          SerCfgWrt  <= '1';  -- Trigger a write operation to the eeprom
          SerCfgDev  <= DATA_IN(2 downto 0);  -- Catch the relevent physical device address.
          DevPhyAddr <= DATA_IN(2 downto 0);  -- and stash it so PAN read back can be done.
        end if;
      else
        SerCfgRd  <= '0';     -- This trigger has done it's job so let it rest.
        SerCfgWrt <= '0';     -- and this little trigger went to market ...
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- FSM to write data to the EEPROM.
------------------------------------------------------------------------
  SerCfgWriteRom : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1' and RstLockOut = '0') then  -- First clock after reset true
        WrtRomActive <= '0';
        WrtLoadFlag  <= '0';
        WrtStartReq  <= '0';
        WrtClkEnable <= '0';            -- Kill the clocks
        CfgWrtState  <= CfgWrtIdle;
      else
        case CfgWrtState is
          when CfgWrtIdle =>            -- LOOKING FOR A CHANCE TO DO SOMETHING
            if (SerCfgWrt = '1') then
              WrtRomActive  <= '1';
              CfgWrtAddr    <= (others => '0');   -- Start from address 0.
              WrtRetryCount <= (others => '0');
              CfgWrtState   <= CfgWrtWordLoop;
            else
              WrtRomActive <= '0';      -- Go show the world we're idle.
              WrtClkEnable <= '0';      -- Kill the clocks
              CfgWrtState  <= CfgWrtIdle;
            end if;
          when CfgWrtWordLoop =>        -- OUTER LOOP FOR WRITE SEQUENCE
            if (CfgWrtAddr < MAX_RAM_ADDR) then
              WrtStartReq <= '1';       -- Send the start request
              CfgWrtState <= CfgWrtSendAddr;
            else
              CfgWrtState <= CfgWrtIdle;  -- All done, go home.
            end if;
          when CfgWrtSendAddr =>  -- WAITING FOR START SEQUENCE TO FINISH THEN ISSUE ADDRESS BITS
            if (WrtStartReq = '1') then
              WrtStartReq  <= '0';  -- 1 clock delay for process to bite and cancel start request.
              WrtClkEnable <= '1';      -- Start our clocks
              CfgWrtState  <= CfgWrtSendAddr;
            elsif (StartBusy = '0') then  -- Go look for acknowledge
              if (SdaAck = '1') then    -- We have received a good acknowledge.
                WrtHighByte <= '0';     -- Initialize our byte pointer
                WrtLoadData <= CfgWrtAddr(6 downto 0) & '0';  -- Set up the current address
                WrtLoadFlag <= '1';     -- and load the address data.
                CfgWrtState <= CfgWrtAddrAck;
              elsif (WrtRetryCount < MAX_WRT_RETRY) then
                WrtRetryCount <= WrtRetryCount + 1;
                WrtDlyCntr    <= (others => '0');
                CfgWrtState   <= CfgWrtDelay;
              else
                WrtStopReq  <= '1';
                CfgWrtState <= CfgWrtSendStop;    -- Give up here ...
              end if;
            else
              CfgWrtState <= CfgWrtSendAddr;
            end if;
          when CfgWrtDelay =>           -- DELAY TILL TIME FOR A RETRY.
            WrtClkEnable <= '0';        -- Kill our clocks for a while
            if (WrtDlyCntr < RETRY_PERIOD) then
              WrtDlyCntr  <= WrtDlyCntr + 1;
              CfgWrtState <= CfgWrtDelay;
            else
              CfgWrtState <= CfgWrtWordLoop;      -- Go back and try again.
            end if;
          when CfgWrtAddrAck =>         -- WAIT FOR ADDRESS ACKNOWLEDGE
            if (WrtLoadFlag = '1') then
              WrtLoadFlag <= '0';  -- 1 clock delay for process to bite and cancell request
              CfgWrtState <= CfgWrtAddrAck;
            elsif (WriteBusy = '0') then  -- Go look for acknowledge when write process stopped
              if (SdaAck = '1') then    -- We have received a good acknowledge.
                CfgWrtState <= CfgWrtDataByte;
              else
                WrtStopReq  <= '1';
                CfgWrtState <= CfgWrtSendStop;    -- Bad one ... Error !!    
              end if;
            else
              CfgWrtState <= CfgWrtAddrAck;
            end if;
          when CfgWrtDataByte =>
            if (WrtHighByte = '0') then   -- If we're on the first byte
              WrtLoadData <= loc_data(7 downto 0);  -- Set up the current data byte
              WrtLoadFlag <= '1';       -- and load the address data.
              WrtHighByte <= '1';       -- Increment the byte counter.
              CfgWrtState <= CfgWrtDataWait;
            else
              WrtLoadData <= loc_data(15 downto 8);  -- Set up the current data byte
              WrtLoadFlag <= '1';       -- and load the address data.
              WrtHighByte <= '0';       -- Increment the byte counter.
              CfgWrtState <= CfgWrtDataWait;
            end if;
          when CfgWrtDataWait =>
            if (WrtLoadFlag = '1') then
              WrtLoadFlag <= '0';
              CfgWrtState <= CfgWrtDataWait;
            elsif (WriteBusy = '0') then
              if (SdaAck = '1') then    -- We have received a good acknowledge.
                if (WrtHighByte = '0') then
                  CfgWrtAddr  <= CfgWrtAddr + '1';  -- Increment the word pointer.
                  WrtStopReq  <= '1';   -- Ask for a stop sequence
                  CfgWrtState <= CfgWrtSendStop;
                else
                  CfgWrtState <= CfgWrtDataByte;
                end if;
              else
                CfgWrtState <= CfgWrtIdle;        -- Bad one ... Error !!    
              end if;
            else
              CfgWrtState <= CfgWrtDataWait;
            end if;
          when CfgWrtSendStop =>
            if (WrtStopReq = '1') then
              WrtStopReq  <= '0';
              WrtDlyCntr  <= (others => '0');     -- Get ready for a delay.
              CfgWrtState <= CfgWrtSendStop;
            elsif (StopBusy = '0') then
              CfgWrtState <= CfgWrtDelay;
            else
              CfgWrtState <= CfgWrtSendStop;
            end if;
          when others =>
            CfgWrtState <= CfgWrtIdle;
        end case;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- FSM to read out the EEPROM.
------------------------------------------------------------------------
  SerCfgReadRom : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1' and RstLockOut = '0') then  -- First clock after reset true
        RdRomSuccess <= '0';
        RdRomActive  <= '0';
        RdLoadFlag   <= '0';
        RdClkEnable  <= '0';            -- Kill the clocks
        RdStartReq   <= '0';            -- Kill all the service requests
        RdStopReq    <= '0';
        RdEnableFlag <= '0';            -- Shut down the read process.
        CfgRdState   <= CfgRdIdle;
      else
        case CfgRdState is
          when CfgRdIdle =>             -- LOOKING FOR A CHANCE TO DO SOMETHING
            if (SerCfgRd = '1') then
              RdRomActive         <= '1';
              RdRomSuccess        <= '0';
              CfgRdAddr           <= (others => '0');  -- Begin at shadow ram address zero.
              RdClkEnable         <= '1';  -- Start the clocks
              RdStartReq          <= '1';  -- Send the start request
              RdSndStartWriteFlag <= '1';  -- First time through we want to write enable the eeprom
              CfgRdState          <= CfgRdSendAddr;    -- Go wait for results
            else
              RdRomActive <= '0';       -- Go show the world we're idle.
              RdClkEnable <= '0';       -- Kill the clocks
              CfgRdState  <= CfgRdIdle;
            end if;
          when CfgRdSendAddr =>  -- WAIT TO SEND OUT THE ADDRESS TO ACCESS IN EEPROM.
            if (RdStartReq = '1') then  -- First clock cycle at this state
              RdStartReq <= '0';  -- So wait for the output to rationalize and kill the load request meantime.
              CfgRdState <= CfgRdSendAddr;
            elsif (StartBusy = '0') then  -- Else wait for the eight bits of address to go out
              if (SdaAck = '1') then    -- We have received a good acknowledge.
                RdLoadFlag <= '1';      -- and load the address data.
                CfgRdState <= CfgRdAddrAck;
              else
                RdStopReq  <= '1';    -- No acknowledge, just go die gracefully
                CfgRdState <= CfgRdSendStop;      -- by sending stop.
              end if;
            else
              CfgRdState <= CfgRdSendAddr;  -- Stay here till start and select work is done
            end if;
          when CfgRdAddrAck =>  -- DO WE RECEIVE AN ACKNOWLEDGE FOR THE ADDRESS ?
            if (RdLoadFlag = '1') then  -- Have we just arrived
              RdLoadFlag <= '0';
              CfgRdState <= CfgRdAddrAck;
            elsif (WriteBusy = '0') then  -- Go look for acknowledge when write process stopped
              if (SdaAck = '1') then    -- We have received a good acknowledge.
                RdStartReq          <= '1';  -- Send the restart request
                RdSndStartWriteFlag <= '0';  -- This time through we want to read enable the eeprom
                CfgRdState          <= CfgRdReStart;
              else
                RdStopReq  <= '1';    -- No acknowledge, just go die gracefully
                CfgRdState <= CfgRdSendStop;      -- by sending stop.
              end if;
            else
              CfgRdState <= CfgRdAddrAck;
            end if;
          when CfgRdReStart =>          -- WAIT FOR THE RESTART ACKNOWLEDGE.
            if (RdStartReq = '1') then  -- First clock cycle at this state
              RdStartReq <= '0';  -- So wait for the output to rationalize and kill the load request meantime.
              CfgRdState <= CfgRdReStart;
            elsif (StartBusy = '0') then  -- Else wait for the eight bits of address to go out
              if (SdaAck = '1') then    -- We have received a good acknowledge.
                RdEnableFlag <= '1';    -- Begin down the read process.
                FirstFlag    <= '1';    -- Signal that we've (only) just begun.
                RdHighByte   <= '0';    -- Zero out the byte counter
                AckFlag      <= '1';    -- Set up for normal acknowledge. 
                CfgRdState   <= CfgRdRdDat1;
              end if;
            else
              CfgRdState <= CfgRdReStart;
            end if;

          when CfgRdRdDat1 =>  -- NOW WE BEGIN TO READ IN DATA. SET UP THE PROCESS FOR 254 BYTES TO BE READ.
            SerCfgStoreWrt <= '0';  -- Reset any write strobe from previous cycle.
            if (CfgRdAddr < MAX_RAM_ADDR) then
              if (RdDataRdy = '1') then  -- We have a word .. Stash it in the appropriate place
                if (RdHighByte = '0') then
                  SerCfgStoreData(7 downto 0) <= RdByteValue;
                  RdHighByte                  <= '1';
                  if (FirstFlag = '0') then
                    CfgRdAddr <= CfgRdAddr + 1;
                  end if;
                else
                  SerCfgStoreData(15 downto 8) <= RdByteValue;
                  RdHighByte                   <= '0';
                  SerCfgStoreWrt               <= '1';  -- set the write strobe to write to shadow ram.    
                  FirstFlag                    <= '0';  -- allow address increments after we have the first word captured.
                end if;
              end if;
              CfgRdState <= CfgRdRdDat1;
            else
              AckFlag    <= '0';        -- No acknowledge on this last byte.
              CfgRdState <= CfgRdRdDat2;
            end if;
          when CfgRdRdDat2 =>  -- CLEAN UP THE READ PROCESS IN A GOOD WAY
            if (RdDataRdy = '1') then  -- finish reading the last byte to be read with no acknowledge sent
              RdEnableFlag <= '0';      -- Shut down the read process.
              RdRomSuccess <= '1';      -- Set the shadow ram data valid bit.
              RdStopReq    <= '1';      -- that the serial data be loaded.
              CfgRdState   <= CfgRdSendStop;
            else
              CfgRdState <= CfgRdRdDat2;
            end if;
          when CfgRdSendStop =>
            if (RdStopReq = '1') then
              RdStopReq  <= '0';
              CfgRdState <= CfgRdSendStop;
            elsif (StopBusy = '0') then
              CfgRdState <= CfgRdIdle;
            else
              CfgRdState <= CfgRdSendStop;
            end if;
          when others =>
            CfgRdState <= CfgRdIdle;
        end case;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- Start + Select sequence generator for I2C bus
------------------------------------------------------------------------
  SendStartReq <= WrtStartReq or RdStartReq;

  CfgSendStartProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1' and RstLockOut = '0') then
        StartBusy     <= '0';
        StrtLoadFlag  <= '0';
        StrtClkEnable <= '0';           -- and clock resources
        StrtState     <= StrtIdle;
      else
        case StrtState is
          when StrtIdle =>              -- WAITING FOR BUSINESS
            if (SendStartReq = '1') then
              StartBusy     <= '1';
              StrtClkEnable <= '1';     -- Start the clocks
--                            StrtLoadData <= (others => '0');   -- Send the start bit by requesting 
--                            StrtLoadFlag <= '1';               -- that the serial data be loaded.
              StrtState     <= StrtSync;
            else
              StartBusy     <= '0';
              StrtClkEnable <= '0';     -- Kill the clocks
              StrtState     <= StrtIdle;
            end if;
          when StrtSync =>
--                        StrtLoadFlag <= '0';
            if (RdStrobe = '1') then
              StrtLoadData <= (others => '0');  -- Send the start bit by requesting 
              StrtLoadFlag <= '1';      -- that the serial data be loaded.
              StrtState    <= StrtStart;
            else
              StrtState <= StrtSync;
            end if;
          when StrtStart =>             -- WAITING FOR START SEQUENCE TO FINISH
            StrtLoadFlag <= '0';        -- Kill the load request.
            if (ClkPhase0 = '1') then   -- We've got the start sequence correct
              StrtLoadData <= TYPE_IDENT & SerCfgDev & not WriteFlag;  -- Load the select code in first.
              StrtLoadFlag <= '1';
              StrtState    <= StrtSendSlct;
            else
              StrtState <= StrtStart;
            end if;
          when StrtSendSlct =>  -- WAIT UNTIL ALL DEVICE SELECT BITS HAVE BEEN WRITTEN and ACKNOWLEDGE READ.
            if (StrtLoadFlag = '1') then
              StrtLoadFlag <= '0';
            elsif (WriteBusy = '0') then
              StrtState <= StrtIdle;
            else
              StrtState <= StrtSendSlct;
            end if;
          when others =>
            StrtState <= StrtIdle;
        end case;
      end if;
    end if;
  end process;

  CfgSendStartWrtFlagProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (WrtStartReq = '1') then
        WriteFlag <= '1';               --- WrtSndStartWriteFlag;
      elsif (RdStartReq = '1') then
        WriteFlag <= RdSndStartWriteFlag;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- Stop sequence generator for I2C bus
------------------------------------------------------------------------
  SendStopReq <= WrtStopReq or RdStopReq;

  CfgSendStopProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1' and RstLockOut = '0') then
        StopBusy     <= '0';
        StpLoadFlag  <= '0';
        StpClkEnable <= '0';            -- and clock resources
        SndStopState <= SndStopIdle;
      else
        case SndStopState is
          when SndStopIdle =>
            StpLoadFlag <= '0';
            if (SendStopReq = '1') then
              StopBusy     <= '1';
              StpClkEnable <= '1';
              SndStopState <= SndStopStart;
            else
              StopBusy     <= '0';
              StpClkEnable <= '0';
              SndStopState <= SndStopIdle;
            end if;
          when SndStopStart =>
            if (ClkPhase0 = '1') then
              StpLoadData  <= (others => '0');  -- Send the stop bit by requesting
              StpLoadFlag  <= '1';
              SndStopState <= SndStopStop;
            else
              SndStopState <= SndStopStart;
            end if;
          when SndStopStop =>
            StpLoadFlag <= '0';         -- Cancel the load flag.
            if (RdStrobe = '1') then    -- Wait for the appropriate moment
              StpLoadData  <= (others => '1');  -- Send the stop bit now after the clock has been high for > 600ns. 
              StpLoadFlag  <= '1';      -- Load it.
              SndStopState <= SndStopIdle;  -- Go do it all again ... someday !    
            else
              SndStopState <= SndStopStop;
            end if;
          when others =>
            SndStopState <= SndStopIdle;
        end case;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- Clock generator for read / write FSMs
------------------------------------------------------------------------
  ClkPhase0     <= not (SerCfgClk(0) or SerCfgClk(1));   -- Clock state low 
  ClkPhase1     <= (not SerCfgClk(1)) and SerCfgClk(0);  -- Clock state high
  ClkPhase2     <= (not SerCfgClk(0)) and SerCfgClk(1);  -- Clock state low
  ClkEnableFlag <= StpClkEnable or StrtClkEnable or RdClkEnable or WrtClkEnable;

  CfgClkGenProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (ClkEnableFlag = '1') then
        if (ClkCntr = CLK_PERIOD) then
          ClkCntr <= (others => '0');   -- Time we changed the clock state
          if (SerCfgClk(1) = '1') then  -- test the clock phase for end of cycle
            SerCfgClk <= (others => '0');
          else
            SerCfgClk <= SerCfgClk + 1;  -- Generate the clock triplet
          end if;
        else
          if (ClkPhase1 = '1' and ClkCntr = RD_STROBE) then
            RdStrobe <= '1';
          else
            RdStrobe <= '0';
          end if;
          ClkCntr <= ClkCntr + 1;
        end if;
      else
        ClkCntr   <= (others => '0');  -- We're initially sync'ed to the start of any FSM through RdRomActive.
        SerCfgClk <= "01";              -- Initial clock state is high.
        RdStrobe  <= '0';  -- Cancell any hung read strobe ... Murphy lurks here.
      end if;
    end if;
  end process;

  SCL <= SerCfgClk(0);                  -- Output the clock state.

------------------------------------------------------------------------
-- Timing and management for parallel to serial converter to SDA
------------------------------------------------------------------------
  LoadData <= WrtLoadData when (WrtLoadFlag = '1') else
              x"00"        when (RdLoadFlag = '1')   else
              StpLoadData  when (StpLoadFlag = '1')  else
              StrtLoadData when (StrtLoadFlag = '1') else
              x"FF";

  LoadFlag  <= RdLoadFlag or WrtLoadFlag or StpLoadFlag or StrtLoadFlag;
  SdaEnable <= WrtSdaEnable or AckSdaEnable;
  SDA_ENAB  <= SdaEnable;

  SDA_OUT <= not AckFlag when (AckSdaEnable = '1') else SerWrtData(7);  -- Output the serial data to the world.

  CfgWrtDataProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (LoadFlag = '1') then
        SerWrtData <= LoadData;
        BitCntr    <= (others => '0');
        DataArm    <= '0';
      else
        case SerCfgClk is
          when "00" =>
            if (DataArm = '1') then
              SerWrtData <= SerWrtData(6 downto 0) & '1';
              BitCntr    <= BitCntr + 1;
              DataArm    <= '0';
            end if;
          when "01" =>
            null;
          when "10" =>
            DataArm <= '1';
          when others =>
            null;
        end case;
      end if;
    end if;
  end process;

  CfgWrtDataStateProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1' and RstLockOut = '0') then
        WriteBusy    <= '0';
        WrtSdaEnable <= '0';            -- Release hold on sda line
        WrtDataState <= WrtDataIdle;
      else
        case WrtDataState is
          when WrtDataIdle =>
            if (LoadFlag = '1') then
              WriteBusy    <= '1';
              WrtDataState <= WrtDataBusy;
              WrtSdaEnable <= '1';      -- Enable the sda line.
            else
              WriteBusy    <= '0';
              WrtSdaEnable <= '0';      -- Disable the sda line when idle.
              WrtDataState <= WrtDataIdle;
            end if;
          when WrtDataBusy =>
            if (BitCntr(3) = '1') then  -- Ninth bit time ... We're due an acknowledgement bit
              WrtSdaEnable <= '0';  -- Disable the master SDA sig and listen for an acknowledge
              WrtDataState <= WrtDataAcknowledge;
            else
              WrtDataState <= WrtDataBusy;
            end if;
          when WrtDataAcknowledge =>  -- WAIT FOR ACKNOWLEDGE TIME AND SAMPLE SDA
            if (RdStrobe = '1') then  -- Read the acknowledge result in the middle of the clock high pulse
              SdaAck       <= not SDA_IN;
              WrtDataState <= WrtDataEndFrame;
            else
              WrtDataState <= WrtDataAcknowledge;
            end if;
          when WrtDataEndFrame =>       -- WAIT FOR END OF NINTH BIT TIME.
            if (ClkPhase0 = '1') then
              WrtDataState <= WrtDataIdle;
            else
              WrtDataState <= WrtDataEndFrame;
            end if;
          when others =>
            WrtDataState <= WrtDataIdle;
        end case;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- Timing and management for serial to parallel converter from SDA
------------------------------------------------------------------------    
  SerCfgRdData : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RdEnableFlag = '1') then
        if (RdBitCntr < BITS_PER_BYTE) then  -- HAVE WE COLLECTED ALL THE BITS YET ?
          AckSdaEnable <= '0';  -- Release the serial signal to the device
          RdDataRdy    <= '0';          -- Cancel any previous data ready flag.
          RdAckActive  <= '0';  -- and local flag used during acknowledgement.
          if (RdStrobe = '1') then      -- IT'S TIME TO READ THE SDA DATA BIT
            SerRdData(0) <= SDA_IN;  -- Catch the SDA level at the correct time
            ShiftFlag    <= '1';        -- and prepare for a shift opperation.
            RdBitCntr    <= RdBitCntr + 1;   -- Increment the bit counter.
          elsif (ShiftFlag = '1') then  -- WE'VE COLLECTED, NOW SHIFT IT.
            SerRdData <= SerRdData(6 downto 0) & '0';  -- After grabbing the bit, shift it through
            ShiftFlag <= '0';           -- and make sure it's only one shift.
          end if;
        else
          if (ClkPhase0 = '1' and AckSdaEnable = '0') then  -- Okay lets start the Acknowledge to the device.
            AckSdaEnable <= '1';  -- Enable the tristate gate on the SDA line which sets acknowledge true
          elsif (ClkPhase2 = '1' and AckSdaEnable = '1') then
            RdAckActive <= '1';   -- Wait for the 2nd clock phase to pass.
          elsif (ClkPhase0 = '1' and RdAckActive = '1') then
            AckSdaEnable <= '0';
            RdByteValue  <= SerRdData;  -- Latch the data to our correspondents (redundent really, this thing is so slow !)
            SerRdData    <= (others => '0');  -- Should be redundent.
            RdBitCntr    <= (others => '0');  -- Go back and get a another byte
            RdDataRdy    <= '1';  -- Flag our sponsor that the data is ready.
          end if;
        end if;
      else                              -- NOT READING ... BUT BE READY.
        RdDataRdy    <= '0';            -- Hold off the data ready flag.
        ShiftFlag    <= '0';
        SerRdData    <= (others => '0');  -- Start clean ... (Hey mister clean ... Your dirty now too ! NY 2002)
        RdBitCntr    <= (others => '0');
        AckSdaEnable <= '0';            -- Allow others to use the SDA signal.
      end if;
    end if;
  end process;

end Behavioral;

