------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB FIBER FPGA U25
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   PIXEL BUS DATA FIFO
-- VERSION      :   4.61 
------------------------------------------------------------------------
-- FILE NAME    :   PIXEL_DATA_FIFO.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   02/08/2006
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 7.1
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- IMPLEMENTS A 128 DEEP x 64 WIDE FIFO FOR BUFFERING PIXEL AND MESSAGE
-- DATA FROM THE PIXEL DATA BUS. THE 64 BIT INPUT DATA IS MULTIPLEXED ONTO
-- THE FIFO AS 2 x 32 BIT DATA OUTPUT WORD. THIS MODULE CONNECTS 
-- DIRECTLY TO THE FPDP DATA MUX TO SEND DATA TO THE DHE. 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
-- 02/19/2003 PCM  2.1     ORIGINAL VERSION
-- 04/17/2003 PCM  3.0     MODIFIED TO USE PIPELINE WRITES, 48 BIT PIXEL BUS
--                         AND 128 WORDS DEEP TO ACCOMODATE BURSTS OF 256 CHANNELS
-- 04/22/2004 PCM  4.0     REMOVED SEQUENCER CODING METHOD SINCE WE ONLY READ ONE
--                         TYPE OF DATA FROM ANY READ GENERATED BY THE SEQUENCER 
--                         FPGFA (32 BIT READ). ADDED ENABLING SIGNAL FOR PIPELINE REQs
-- 02/14/2005 PCM  4.4     INSERTED CAPABILITY TO SEND TWO 16 BIT PIXEL VALUES FOR EACH 
--                         32 BIT WRITE TO THE FPDP INTERFACE. OPTIMIZED FIFO SIZE FOR
--                         DATA WIDTHS. RENAMED PIPE_REQ_N(7) TO PIPE_WRT_N.
-- 07/14/2005 PCM  4.5     REVERSED PIXEL ORDER IN DOUBLE PIXEL TRANSFERS TO BE SUCH THAT
--                         LOW (EVEN) ORDER PIXELS ARE SENT IN BITS 31:16 OF DBLE XFER.
-- 02/08/2006 PCM  4.61    ADDED PIPELINE_ENBL SIG AS RESET TO FIFO FUNCTIONS- WAS JUST AN INHIBIT
--                         ADDED PIXEL SIMULATION CONTROL PROCESS
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK          : BOARD CLOCK, EVERYTHING HAPPENS ON RISING EDGE
-- RST          : LOCAL RESET TO FIFO, CLEARS FIFO CONTENTS
-- DATA_IN      : 64 BIT PIXEL BUS INPUT
-- DATA_OUT     : 32 BIT OUTPUT TO FPDP DATA MUX
-- SYNC_OUT     : BUFFERED COPY OF SYNC_IN SIGNAL
-- WR_CODE      : CODED WRITE STROBE (SEE IMPLEMENTATION NOTES)
-- PIPELINE_REQ_N : WRITE REQUEST LINES FROM (UP TO) 8 ACQUISITION BOARDS.
-- RD           : READ FROM FIFO OUTPUT
-- DATA_RDY     : DATA AVAILABLE IN FIFO
-- SIM_ENABLE   : ENABLE THE SIMULATION MODE. LOCKS OUT NORMAL PIPELINE MODE
-- SIM_PIXROWS  : NUMBER OF SIMULATED PIXEL ROWS TO SEND
-- SIM_PIXCOLS  : NUMBER OF SIMULATED PIXEL COLUMNS TO SEND
-- SIM_PIXTIME  : TIME BETWEEN PIXEL BURSTS IN UNITS OF 100ns (MINIMUM VALUE IS 5 FOR IR, 8 FOR CCD) 
-- SIM_CHANNELS : NUMBER OF SIUMLATED PIXEL DATA VALUES TO SEND FOR EACH BURST (TOTAL PIXELS ARE SIM_PIXROWS X SIM_PIXCOLS X SIM_PIXCHANNELS)
-- SIM_DELAY    : DELAY AFTER INITIAL TRIGGER BEFORE COMMENCING SIMULATION IN UNITS OF MILLISECONDS.
-- SIM_TRIGGER  : TRIGGER A SIMULATED READOUT
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity PIXEL_DATA_FIFO_V461 is
  port (CLK            : in  std_logic;
        RST            : in  std_logic;
        DATA_IN        : in  std_logic_vector(47 downto 0);
        DATA_OUT       : out std_logic_vector(31 downto 0);
        SEQ_WRT_N      : in  std_logic;
        PIPELINE_REQ_N : in  std_logic_vector(6 downto 0);
        PIPELINE_WRT_N : in  std_logic;
        RD             : in  std_logic;
        DBL_WD_XFER    : in  std_logic;
        PIPELINE_ENBL  : in  std_logic;
        DATA_RDY       : out std_logic;
        SYNC_OUT       : out std_logic;
        SIM_ENABLE     : in  std_logic;
        SIM_PIXROWS    : in  std_logic_vector(15 downto 0);
        SIM_PIXCOLS    : in  std_logic_vector(15 downto 0);
        SIM_PIXTIME    : in  std_logic_vector(7 downto 0);
        SIM_CHANNELS   : in  std_logic_vector(6 downto 0);
        SIM_DELAY      : in  std_logic_vector(7 downto 0);
        SIM_TRIGGER    : in  std_logic
        );
end PIXEL_DATA_FIFO_V461;

architecture Behavioral of PIXEL_DATA_FIFO_V461 is
------------------------------------------------------------------------
-- Set up a RAM block for the fifo itself
-- Infer a Distributed RAM register block 50 bits wide x 128 deep
-- Bit (49) is used to flag non pixel data bus xfers which require sync high and fifo data width is 32 bits.
-- Bit (48) is used to flag 48 bit xfers of pixel data i.e. fifo significant data width is 48 bits. 
------------------------------------------------------------------------
  type   fifo_ram_type is array (127 downto 0) of std_logic_vector (49 downto 0);
  signal fifo_ram                        : fifo_ram_type;
  signal fifo_wrt_addr, fifo_rd_addr     : std_logic_vector(6 downto 0);
  signal fifo_empty, first_read, flag_48 : std_logic;
  signal fifo_reg                        : std_logic_vector(49 downto 0);
  signal fifo_wrt                        : std_logic;
  signal wrtpipe                         : std_logic;

------------------------------------------------------------------------
-- Kludge to overcome XilinxBaseX compiler fobia against mixed vectors !!
------------------------------------------------------------------------
  signal Loc_Dat : std_logic_vector(49 downto 0);

------------------------
-- Simulation signals
------------------------
  constant MilliSecValue              : std_logic_vector(15 downto 0) := x"9C3F";  -- 40,000 counts of a 40 MHz clock for each millisecond
  constant PixTimeValue               : std_logic_vector(1 downto 0)  := "11";  -- 4 counts of a 40 MHz clock for each 100 ns
  type     SimStateType is (SimIdle, SimDelay, SimRowLoop, SimColLoop);
  signal   SimState                   : SimStateType;
  signal   SimRowCnt, SimRowCtr       : std_logic_vector(15 downto 0);
  signal   SimColCnt, SimColCtr       : std_logic_vector(15 downto 0);
  signal   SimPixDlyCnt, SimPixDlyCtr : std_logic_vector(7 downto 0);
  signal   SimChanCnt, SimChanCtr     : std_logic_vector(6 downto 0);
  signal   SimChanData                : std_logic_vector(6 downto 0);
  signal   SimPixData                 : std_logic_vector(15 downto 0);
  signal   SimTrigger                 : std_logic;
  signal   SimBrstTrigger             : std_logic;  -- Triggers a burst of n pixels to be sent to PAN: n = channels
  signal   MilliSecFlag               : std_logic;  -- Counts milliseconds after SimState goes non-idle
  signal   MilliSecCtr                : std_logic_vector(15 downto 0);
  signal   SimDlyCtr                  : std_logic_vector(7 downto 0);  -- Used for initial post trigger delay
  signal   PixTimeCtr                 : std_logic_vector(1 downto 0);
  signal   PixTimeFlag                : std_logic;
begin

------------------------------------------------------------------------
-- Static assignments
------------------------------------------------------------------------
  DATA_RDY <= not fifo_empty;

  wrtpipe <= PIPELINE_ENBL
             and not PIPELINE_WRT_N
             and not (PIPELINE_REQ_N(0)
                      and PIPELINE_REQ_N(1)
                      and PIPELINE_REQ_N(2)
                      and PIPELINE_REQ_N(3)
                      and PIPELINE_REQ_N(4)
                      and PIPELINE_REQ_N(5)
                      and PIPELINE_REQ_N(6));

------------------------------------------------------------------------
-- FIFO Write process and address manipulation
------------------------------------------------------------------------
  fifo_mux : process (CLK)
  begin
    if (rising_edge (CLK)) then
      if (SIM_ENABLE = '1' and SimTrigger = '1') then
        if (DBL_WD_XFER = '1') then  -- Truncate data to 2 x 16 bit values and
                                     -- indicate 32 bit data significance.
          fifo_reg <= "00"
                      & x"0000"
                      & SimChanData
                      & '1'
                      & SimPixData(7 downto 0)
                      & SimChanData
                      & '0'
                      & SimPixData(7 downto 0);
        else
          fifo_reg <= "01"
                      & SimChanData
                      & '1'
                      & SimPixData
                      & SimChanData
                      & '0'
                      & SimPixData;
        end if;
        fifo_wrt <= '1';
      elsif (wrtpipe = '1') then
        if (DBL_WD_XFER = '1') then  -- Truncate data to 2 x 16 bit values and
                                     -- indicate 32 bit data significance.
          fifo_reg <= "00"
                      & x"0000"
                      & DATA_IN(39 downto 24)
                      & DATA_IN(15 downto 0);
        else
          fifo_reg <= "01" & DATA_IN(47 downto 0);
        end if;
        fifo_wrt <= '1';
      elsif (SEQ_WRT_N = '0') then
        fifo_reg <= "10" & x"0000" & DATA_IN(31 downto 0);
        fifo_wrt <= '1';
      else
        fifo_wrt <= '0';
      end if;
    end if;
  end process;


  fifo_write : process (CLK)
  begin
    if (rising_edge(CLK)) then
      if (RST = '1' or PIPELINE_ENBL = '0') then
        fifo_wrt_addr <= (others => '0');
      elsif (fifo_wrt = '1') then
        fifo_ram(conv_integer(fifo_wrt_addr)) <= fifo_reg;
        fifo_wrt_addr                         <= fifo_wrt_addr + 1;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- FIFO Read process and address manipulation
------------------------------------------------------------------------

  fifo_rd : process (CLK)
  begin
    if (rising_edge(CLK)) then
      if (RST = '1' or PIPELINE_ENBL = '0') then
        fifo_rd_addr <= (others => '0');
        first_read   <= '1';
      elsif (RD = '1' and fifo_empty = '0') then
        if (flag_48 = '1' and first_read = '1') then
          first_read <= '0';
        else
          fifo_rd_addr <= fifo_rd_addr + 1;
          first_read   <= '1';
        end if;
      end if;
    end if;
  end process;

  Loc_Dat <= fifo_ram(conv_integer(fifo_rd_addr));
  flag_48 <= Loc_Dat(48);


  DATA_OUT <= Loc_Dat(31 downto 0) when (flag_48 = '0') else
              x"00" & Loc_Dat(23 downto 0) when (first_read = '1') else
              x"00" & Loc_Dat(47 downto 24);

  SYNC_OUT <= Loc_Dat(49);

  fifo_sigs : process (CLK)
  begin
    if (falling_edge(CLK) and first_read = '1') then
      if (fifo_rd_addr = fifo_wrt_addr) then
        fifo_empty <= '1';
      else
        fifo_empty <= '0';
      end if;
    end if;
  end process;

----------------------------------------------------------
--      SIMULATION TIMING AND PIXEL COUNT ENGINE.       --
-- WHEN TRIGGERED, PRODUCES THE TIMING AND TOTAL COUNT  --
-- OF TRIGGERS TO THE PIXEL BURST ENGINE TO SIMULATE A  --
-- DETECTOR READOUT                                     --
----------------------------------------------------------
  SimPixTimeProc : process(CLK, RST, PIPELINE_ENBL, SIM_ENABLE)
  begin
    -- Asynchronous reset when conditions are not clear
    if (RST = '1' or PIPELINE_ENBL = '0' or SIM_ENABLE = '0') then
      SimRowCtr      <= (others => '0');
      SimColCtr      <= (others => '0');
      SimBrstTrigger <= '0';
      SimState       <= SimIdle;
    elsif (rising_edge(CLK)) then
      case SimState is
        when SimIdle =>
          if (SIM_TRIGGER = '1') then
            SimDlyCtr    <= SIM_DELAY;  -- Latch in the initial values before
                                        -- we send data
            SimRowCnt    <= SIM_PIXROWS;
            SimColCnt    <= SIM_PIXCOLS;
            SimPixDlyCnt <= SIM_PIXTIME;
            SimChanCnt   <= SIM_CHANNELS;
            SimState     <= SimDelay;
          else
            SimState <= SimIdle;
          end if;
        when SimDelay =>
          if (SimDlyCtr = x"00") then
            SimRowCtr  <= SimRowCnt;
            SimPixData <= (others => '0');
            SimState   <= SimRowLoop;
          elsif(MilliSecFlag = '1') then
            SimDlyCtr <= SimDlyCtr - 1;
            SimState  <= SimDelay;
          else
            SimState <= SimDelay;
          end if;
        when SimRowLoop =>
          if(SimRowCtr = x"0000") then
            SimState <= SimIdle;
          else
            SimColCtr    <= SimColCnt;
            SimPixDlyCtr <= SimPixDlyCnt;
            SimState     <= SimColLoop;
          end if;
        when SimColLoop =>
          if(SimColCtr = x"0000" and SimPixDlyCtr = x"00") then
            SimBrstTrigger <= '0';
            SimRowCtr      <= SimRowCtr - 1;
            SimState       <= SimRowLoop;
          elsif (SimPixDlyCtr = x"00") then
            SimBrstTrigger <= '1';
            SimColCtr      <= SimColCtr - 1;
            SimPixData     <= SimPixData + 1;
            SimPixDlyCtr   <= SimPixDlyCnt;
            SimState       <= SimColLoop;
          else
            SimBrstTrigger <= '0';
            if(PixTimeFlag = '1') then
              SimPixDlyCtr <= SimPixDlyCtr - 1;
            end if;
            SimState <= SimColLoop;
          end if;
        when others =>
          SimState <= SimIdle;
      end case;
    end if;
  end process;

  MilliSecondTimerProc : process(CLK, RST, SimState)
  begin
    if (RST = '1' or SimState = SimIdle) then
      MilliSecCtr  <= (others => '0');
      MilliSecFlag <= '0';
    elsif (rising_edge(CLK)) then
      if (MilliSecCtr = MilliSecValue) then
        MilliSecFlag <= '1';
        MilliSecCtr  <= (others => '0');
      else
        MilliSecFlag <= '0';
        MilliSecCtr  <= MilliSecCtr + 1;
      end if;
    end if;
  end process;

  PixelTimerProc : process(CLK, RST, SimState)
  begin
    if (RST = '1' or SimState = SimIdle or SimState = SimRowLoop) then
      PixTimeCtr  <= (others => '0');
      PixTimeFlag <= '0';
    elsif (rising_edge(CLK)) then
      if (PixTimeCtr = PixTimeValue) then
        PixTimeFlag <= '1';
        PixTimeCtr  <= (others => '0');
      else
        PixTimeFlag <= '0';
        PixTimeCtr  <= PixTimeCtr + 1;
      end if;
    end if;
  end process;
----------------------------------------------------------
--           SIMULATION PIXEL BURST ENGINE.             --
-- PRODUCE SIM_CHAN WORTH OF PIXEL DATA EVERYTIME THAT  --
-- SIMULATE TRIGGER IS TRUE. EACH 24 BIT PIXEL DATA IS  --
-- PREFIXED WITH THE 8 BIT CHANNEL NUMBER AND A UNIQUE  --
-- 16 BIT PIXEL COUNT NUMBER                            -- 
----------------------------------------------------------
  SimPixBurstProc : process(CLK, RST, PIPELINE_ENBL, SIM_ENABLE)
  begin
    -- Asynchronous reset when conditions are not clear
    if (RST = '1' or PIPELINE_ENBL = '0' or SIM_ENABLE = '0') then
      SimChanCtr <= (others => '0');
      SimTrigger <= '0';
    elsif (rising_edge(CLK)) then
      if (SimChanCtr = "0000000") then  -- At end of simulated burst of pixels
                                        -- or after reset
        SimTrigger <= '0';
        if (SimBrstTrigger = '1') then  -- If we're not busy then look for a
                                        -- trigger
          SimChanCtr  <= SimChanCnt;
          SimChanData <= (others => '1');
        end if;
      else
        SimTrigger  <= '1';
        SimChanCtr  <= SimChanCtr - 1;  -- If we are simulating, keep the
                                        -- trigger on for the correct number of
                                        -- pixels
        SimChanData <= SimChanData + 1;
      end if;
    end if;
  end process;


end Behavioral;
