------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC
-- DEVICE ID    :   U22 (IR ACQ FPGA)
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   Clock Divider 
-- VERSION      :   4.2 
------------------------------------------------------------------------
-- FILE NAME    :   Clock_Divide.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/14/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- The module based upon the select signals divides the system clock 
-- 00 SeqClk : SYSCLK / 1 = 20 MHz 
-- 01 SeqClk : SYSCLK / 2 = 10 MHz
-- 10 SeqClk : SYSCLK / 4 =  5 MHz
-- 11 SeqClk : SYSCLK / 8 =  2.5 MHz
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE       AUTHOR  VERSION  MODIFICATIONS
-- 03/12/2003  KAVI     1.0     ORIGINAL VERSION
-- 07/11/2003   KC              2.0             Changed the Implementation to gated clock in 
--                              order to minimize clock skew
-- 05/14/2004  PCM      4.2     ADDED CLOCK SYNCHRONIZATION, CLEANED UP THE CODE.               
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLKIN        : System Clock 40 MHz.
-- RATIO    : Clock Divde input select signals 
-- CLKOUT   : Divided sequencer clock output
-- SYNC     : rising edge syncs the clocks.
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Clock_Divide is
    Port ( CLKIN    : in  std_logic;
           reset    : in  std_logic;
         RATIO  : in  std_logic_vector(1 downto 0);
         CLKOUT : out std_logic;
         SYNC   : in  std_logic
         );
end Clock_Divide;

architecture Behavioral of Clock_Divide is
  signal DV       : std_logic_vector(3 downto 0);
  signal DS       : std_logic_vector (3 downto 0);
  signal SyncLock : std_logic;
  signal SyncClk  : std_logic;
begin

  SyncClk <= (SYNC xor SyncLock) and SYNC;  -- Detect the rising edge

--      A clock divider with synchronous clear 
  ClkDivideProc : process (CLKIN)
  begin
    if (rising_edge (CLKIN)) then
            if (SyncClk = '1' or reset='1') then
        DV     <= (others => '0');
        CLKOUT <= '0';
      elsif (DV >= DS) then
        DV     <= (others => '0');
        CLKOUT <= '1';
      else
        DV     <= DV + 1;
        CLKOUT <= '0';
      end if;
    end if;
  end process;

  ClkSyncProc : process (CLKIN)
  begin
    if (rising_edge(CLKIN)) then
      SyncLock <= SYNC;
    end if;
  end process;

-- clock ratio selects
  DS <= "0001" when (RATIO = "00") else
        "0011" when (RATIO = "01") else
        "0111" when (RATIO = "10") else
        "1111";

end Behavioral;
