------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   SEQUENCER FPGA (U22)
-- PARENT MODULE:   (TOP LEVEL)  
-- MODULE       :   BOARD / FPGA IDENTITY MODULE
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   IdentityModule.vhd
-- AUTHOR       :   PETER MOORE
-- DATE         :   22 APRIL 2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE Webpack 5.2
------------------------------------------------------------------------
-- DESCRIPTION  : PROVIDES THE UNIQUE IDENTIFIERS REQUIRED TO IDENTIFY THE
--                PHYSICAL BOARD AND VERSION OF FPGA / CPLD CODE. THESE
--                DATA CAN BE READ BACK BY THE PAN TO DETERMINE THE SYSTEM
--                CONFIGURATION. ACCESS THESE TERMS AT BOARD ADDRESSES :-
--                BOARD ID = ADDRESS 2
--                FPGA ID  = ADDRESS 3  
--                THESE ADDRESSES ARE FIXED BY THE BOARD INTERFACE LOGIC.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- THE BOARD IDENTIFIER IS 16 BITS SPLIT INTO 4 X 4 BIT NIBBLES CODED AS 
-- INDEPENDENT HEX DIGITS.
-- THE FIRST NIBBLE (15:12) IS THE ORIGINATING ORGANIZATION CODE. 
-- THESE CODES ARE:
--                  0 = TEST OR UNASSIGNED.
--                  1 = NATIONAL OPTICAL ASTRONOMY OBSERVATORY.
--                  2 .. 15 = UNDEFINED (CONTACT NOAO TO RESERVE YOUR CODE).
--
-- THE NEXT TWO NIBBLES (11:4) CONTAIN THE MAJOR PORTION OF THE BOARD FUNCTION 
-- CODE. CURRENT BOARD FUNCTION CODES ARE :-
--      00 = TEST OR UNASSSIGNED.
--      01 .. 49 ARE UNASSIGNED.
--      50 =  MASTER CONTROL MOTHER.            51 = MASTER CONTROL DAUGHTER.
--      52 =  18 CHAN IR ACQ MOTHER 1MHZ.       53 = 36 CHAN IR ACQ MOTHER + DAUGHTER.
--      54 =  16 CHAN IR ACQ 5MHZ.              55 = IR CLOCK AND BIAS.
--      56 =  HI SPEED CCD ACQ.                 57 = CCD CLOCK AND BIAS.
--      58 =  QUOTA MCB.                        59 = QUOTA OTCCD 
--      60 =  ROCKWELL DIGITAL                  61 =
--      62 =                                    63 = 
--      64 =                                    65 =
--      66 =                                    67 =
--      68 =                                    69 =
--      61 .. FF ARE UNASSIGNED.
--
-- THE FINAL NIBBLE (3:0) CONTAINS THE MINOR (REVISION) NUMBER FOR BOARD 
-- FUNCTIONALITY. THIS CODE CAN BE USED TO DISTINGUISH UNIQUE CAPABILITIES 
-- OF MODIFIED BOARDS.
--      0 = ORIGINAL VERSION
--      1 .. 9 ARE UNASSIGNED.
-- 
-- THE CODE IDENTIFIER CONTAINS A UNIQUE PATTERN TO IDENTIFY THE FPGA AND/OR 
-- CPLD CODE VERSION CURRENTLY LOADED. THE PATTERN CAN BE GENERATED IN ANY
-- FORM TO REALIZE THIS FUNCTION. CURRENT IMPLEMENTATION USES THE SAME 4 X
-- 4 NIBBLE CODING SCHEME TO DESCRIBE A TWO DIGIT MAJOR + TWO DIGIT MINOR 
-- VERSION SET. VERSION 00.00 IS CONSIDERED TEST OR UNASSIGNED.
--   
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE  AUTHOR  VERSION  MODIFICATIONS
--
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- BOARD_IDENT_OUT  : 16 BIT UNIQUE BOARD IDENTIFIER FROM MONSOON SPECS.
-- CODE_IDENT_OUT   : 16 BIT IDENTIFIER FOR FPGA / CPLD VERSION.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
-- use IEEE.STD_LOGIC_ARITH.ALL;
-- use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity IdentityModule is
  port (board_ident_out : out std_logic_vector(15 downto 0);
--         detector_ident_out : out std_logic_vector(15 downto 0); -- Future use !!
         code_ident_out : out std_logic_vector(15 downto 0)
         );
end IdentityModule;

architecture Behavioral of IdentityModule is

  constant board_ident : std_logic_vector := x"0190";  -- MASTER CONTROL BOARD PART NUMBER (0400)
  constant code_ident  : std_logic_vector := x"01CE";  -- VERS 4.61

begin

  board_ident_out <= board_ident;
  code_ident_out  <= code_ident;

end Behavioral;
