------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC
-- DEVICE ID    :   U22 (IR ACQ FPGA)
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   Integration REGISTER
-- VERSION      :   2.0 
------------------------------------------------------------------------
-- FILE NAME    :   Int_Reg.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   12/06/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- The module is a 32 bit register with write enable 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :

------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE        AUTHOR  VERSION  MODIFICATIONS
-- 02/21/2003  KAVI    1.0      ORIIGINAL VERSION
-- 12/06/2004  PCM     2.0      ADDED EXTRA CLOCK DELAY BY AND'ING intcntr_run
--                              ON TC TO GIVE THE COMPARITOR TIME TO RESPONDE
-- 11-Dec-2008 JTO              add SYNC reset, cleanup inout outputs
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- DATAIN       : 32 BIT REQUIRED INTEGRATION TIME WRITEN TO REGISTER 
-- WRITE        : WRITE STROBE TO LOAD TIMER REGISTER
-- SYSCLK       : SYSTEM CLOCK (NOMINALLY 40 MHz)
-- RESET    : RESET EVERYTHING
-- ENABLE       : MUST GO HIGH TO INITIATE TIMER, RISING EDGE RESETS COUNTER
-- PAUSE    : PAUSES THE INTEGRATION TIMER 
-- TC       : TERMINAL COUNT, ACTIVE WHEN CNTR >= REQUIRED TIME
-- INTREG   : REQUIRED INTEGRATION TIME SET TO THIS REGISTER
-- INTCTR   : ACTUAL INTEGRATION TIME 
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Int_Reg is
    Port ( DATAIN 	: in std_logic_vector(31 downto 0);
         WRITE  : in    std_logic;
         SYSCLK : in    std_logic;
         RESET  : in    std_logic;
         ENABLE : in    std_logic;
         PAUSE  : in    std_logic;
           TC       : out std_logic;
           INTREG   : out std_logic_vector(31 downto 0);
           INTCTR   : out std_logic_vector(31 downto 0)
         );
end Int_Reg;

architecture Behavioral of Int_Reg is

    signal intreg_reg, intctr_reg : std_logic_vector(31 downto 0);

  signal   prescale_run, intcntr_run, msclk : std_logic;
  signal   prescale_cntr                    : std_logic_vector(15 downto 0);
  constant prescale_val                     : std_logic_vector(15 downto 0) := x"9C40";  -- 1ms resolution on 40MHz sysclk

begin

    TC <= '1' when (INTREG_reg <= INTCTR_reg and intcntr_run = '1') else '0'; 

    intreg <= intreg_reg;
    intctr <= intctr_reg;

------------------------------------------------------------------------
-- LOAD AND RESET FOR REQUIRED INTEGRATION TIMER REGISTER
------------------------------------------------------------------------
  process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1') then
				INTREG_reg <=  (others => '0');
      elsif (WRITE = '1') then
    	      	INTREG_reg <= DATAIN;
      end if;
    end if;
  end process;


------------------------------------------------------------------------
-- PRESCALE COUNTER .. ADJUST PRESCALE_VAL TO DESIRED RESOLUTION
------------------------------------------------------------------------

--	prscale : process (SYSCLK)
--	begin
--		if (rising_edge(SYSCLK)) then
--			msclk <= '0';
--			if (ENABLE = '1') then
--		  		if (prescale_run = '0') then
--					prescale_run <= '1';
--					prescale_cntr <= (others => '0');
--				elsif (PAUSE = '0') then
--					if (prescale_cntr >= prescale_val) then
--						msclk <= '1';
--						prescale_cntr <= (others => '0');
--					else
--						prescale_cntr <= prescale_cntr + 1;
--					end if;
--				end if;
--			else
--				prescale_run <= '0';
--			end if;
--		end if;
--	end process;

  prscale : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
            if (reset='1') then
                msclk <= '0';
                prescale_run <= '0';
                prescale_cntr <= (others => '0');
            else
    
      msclk <= '0';
      if (ENABLE = '1') then
        if (prescale_run = '0') then
          prescale_run  <= '1';
          prescale_cntr <= (others => '0');
        elsif (PAUSE = '0') then
          if (prescale_cntr >= prescale_val) then
            msclk         <= '1';
            prescale_cntr <= (others => '0');
          else
            prescale_cntr <= prescale_cntr + 1;
          end if;
        end if;
      else
        prescale_run <= '0';
      end if;

            end if;
    end if;
  end process;

------------------------------------------------------------------------
-- ACTUAL INTEGRATION TIMER COUNTER
------------------------------------------------------------------------

--	process (SYSCLK)
--	begin
--		if (rising_edge(SYSCLK)) then
--    	   	if (RESET = '1') then
--    		    INTCTR <=  (others => '0');
--			elsif (ENABLE = '1') then
--				if (intcntr_run = '0') then
--					INTCTR <= (others => '0');
--					intcntr_run <= '1';
--				elsif (msclk = '1') then
--					INTCTR <= INTCTR + 1;
--				end if;					
--			else
--				intcntr_run <= '0';
--			end if;
--		end if;
--	end process;

  process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1') then
    		    INTCTR_reg <=  (others => '0');
                intcntr_run <= '0';
      elsif (ENABLE = '1') then
        if (intcntr_run = '0') then
					INTCTR_reg <= (others => '0');
          intcntr_run <= '1';
        elsif (msclk = '1') then
					INTCTR_reg <= INTCTR_reg + 1;
        end if;
      else
        intcntr_run <= '0';
      end if;
    end if;
  end process;

end Behavioral;

