------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   PATTERN GENERATOR
-- VERSION      :   4.2 
------------------------------------------------------------------------
-- FILE NAME    :   PATTERN_GENERATOR.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/19/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- IMPLEMENTS A 32 BIT DOWN COUNTER. COUNTER IS SYNCRONOUSLY LOADABLE AND
-- COUNTS DOWN WHEN CLK_EN IS TRUE AND A START EXPOSURE COMMAND IS RECEIVED. 
-- THE COUNTER STOPS COUNTING AFTER ZERO IS REACHED OR CLK_EN GOES FALSE.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- CLK IS ASSUMED TO BE 40MHZ. A PRESCALER IS USED TO PROVIDE NEW COUNTER
-- DATA EVERY 40 CLOCKS (I.E. EVERY 1us.)
--
------------------------------------------------------------------------
-- REVISION HISTORY :
--    DATE        AUTHOR  VERSION  MODIFICATIONS
-- 02/19/2003  PCM      2.1    ORIGINAL VERSION
-- 07/08/2003  PCM      2.2    REMOVED TC TERMINAL
-- 05/19/2004  PCM      4.2    ADDED ENABLE BIT.
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK      : SYSTEM CLOCK
-- RST      : SYNCRONOUS RESET OF COUNTER TO ZERO
-- LOAD     : LOAD DATA_IN TO COUNTER WHEN TRUE ON RISING EDGE OF CLK
-- STRT_EXP : STROBED TRUE TO ENABLE COUNT DOWN TO BEGIN
-- ENABLE   : GLOBAL ENABLE FLAG TO PERMIT COUNTING.
-- DATA_IN  : 32 BIT SEED VALUE TO COUNTER 
-- DATA_OUT : 32 BIT COUNTER DATA
-- DATA_RDY : TRUE FOR ONE CLK PERIOD AFTER COUNTER VALUE CHANGES
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity PATTERN_GENERATOR is
  port (CLK      : in  std_logic;
        RST      : in  std_logic;
        LOAD     : in  std_logic;
        STRT_EXP : in  std_logic;
        ENABLE   : in  std_logic;
        DATA_IN  : in  std_logic_vector(31 downto 0);
        DATA_OUT : out std_logic_vector(31 downto 0);
        DATA_RDY : out std_logic
        );
end PATTERN_GENERATOR;

architecture Behavioral of PATTERN_GENERATOR is

  signal cntr                  : std_logic_vector(31 downto 0);  -- Actual counter register
  signal prsc                  : std_logic_vector(5 downto 0);  -- Prescaler counter
  signal s_clk, cnt_en, cnt_go : std_logic;

begin

------------------------------------------------------------------------
-- Prescaler counter. counts from 0 up to 39 and issues s_clk
------------------------------------------------------------------------
  PreScl : process (CLK)
  begin
    if (rising_edge(CLK)) then
      s_clk <= '0';
      if (RST = '1' or LOAD = '1') then
        prsc <= (others => '0');
      elsif (cnt_go = '1') then
        if (prsc = "100111") then
          s_clk <= '1';
          prsc  <= (others => '0');
        else
          prsc <= prsc + 1;
        end if;
      end if;
    end if;
  end process;

------------------------------------------------------------------------
-- Data products resolved here
------------------------------------------------------------------------
  cnt_en   <= '0' when (cntr = x"00000000") else ENABLE;
  DATA_OUT <= cntr;

------------------------------------------------------------------------
-- Actual 32 bit counter process
------------------------------------------------------------------------
  CntDwn : process (CLK)
  begin
    if (rising_edge(CLK)) then
      DATA_RDY <= '0';
      if (RST = '1') then
        cntr   <= (others => '0');
        cnt_go <= '0';
      elsif (LOAD = '1') then
        cntr   <= DATA_IN;
        cnt_go <= '0';
      elsif (STRT_EXP = '1') then
        cnt_go <= '1';
      elsif (cnt_go = '1' and cnt_en = '1') then
        if (s_clk = '1') then
          cntr     <= cntr - 1;
          DATA_RDY <= '1';
        end if;
      end if;
    end if;
  end process;
  
end Behavioral;
