------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MCB
-- DEVICE ID    :   MCB SEQUENCER FPGA
-- PARENT MODULE:   TOP LEVEL DESIGN  
-- MODULE       :   PERSONALITY MODULE
-- VERSION      :   4.4 
------------------------------------------------------------------------
-- FILE NAME    :   PERSONALITYMODULE.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   02/14/2005
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE Webpack 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- THIS MODULE PROVIDES THE LOGIC MECHANISM TO INTERFACE THE SEQUENCER
-- BUS INTERFACE TO ANY PARTICULAR VIRTEX FPGA DESIGN. 
-- IT PROVIDES FOR LOCAL ADDRESS DECODE LOGIC AND A SIMPLE DATA SELECTION
-- MUX TO PERSONALIZE YOUR DESIGN TO A PARTICULAR FPGA LOGIC IMPLEMENTATION 
-- TO THE SEQUENCER INTERFACE LOGIC BLOCK.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- FIXED ADDRESS ASSIGNMENTS
-- LED INDICATOR SELECT PORT     = 0xFFF9 (3 BITS WRITE - 3 BITS READ)
-- CIRCUIT BOARD SERIAL NUMBER   = 0xFFFA (NO WRITE - 48 BITS READ)
-- CIRCUIT BOARD TEMPERATURE     = 0xFFFB (NO WRITE - 16 BITS READ)
-- CONTROL REGISTER ADDRESS      = 0xFFFC (16 BIT WRITE - 16 BIT READ)
-- STATUS REGISTER ADDRESS       = 0XFFFD (NO WRITE - 16 BIT READ) 
-- BOARD ID REGISTER ADDRESS     = 0xFFFE (NO WRITE - 16 BIT READ)
-- CODE ID REGISTER ADDRESS      = 0xFFFF (NO WRITE - 16 BIT READ)
-- SEQUENCER FPGA SPECIFIC ADDRESS MAP
-- REQUESTED INTEGRATION TIME    = 0x0000 (32 BIT WRITE - 32 BIT READ)
-- TEST DATA GENERATOR           = 0x0001 (32 BITS WRITE - 32 BITS READ)
-- CLOCK ENABLE REGISTER         = 0x0100 (7 BITS WRITE - 16 BITS READ)
-- ACTUAL INTEGRATION TIME       = 0x0101 (NO WRITE - 32 BIT READ)
-- SEQUENCER MPU COMMAND REG     = 0x0102 (4 BITS WRITE - 16 BIT READ)
-- SEQUENCER MPU LOOP REGISTERS  = 0x0110 => 0x011F (16 BITS WRITE - 16 BITS READ)
-- SERIAL CONFIGURATION DATA     = 0x0200 => 0x027F (16 BITS WRITE - 16 BITS READ) (NOTE 1)
-- DHE CLOCK SYNC PHASE CONTROL  = 0x0280 (8 BITS WRITE - 8 BITS READ) (NOTE 1)
-- PIXEL FPGA LED INDICATOR PORT = 0x0281 (3 BIT WRITE - 3 BITS READ) (NOTE 1)
-- SYNC LINK SERIAL DATA PORT    = 0x0282 (16 BITS WRITE - 16 BITS READ) (NOTE 1, 2)
-- HIGH SPEED SERIAL LINK PORT   = 0x0283 (16 BITS WRITE - 16 BITS READ) (NOTE 1, 2)
-- PIXEL FPGA STATUS PORT        = 0x0284 (NO WRITE - 16 BITS READ) (NOTE 1, 2)
-- EXTERNAL DATA PORT            = 0x0285 (4 BITS WRITE - 4 BITS READ) (NOTE 1, 2)
-- TEMP / DAC CHANNEL PORT       = 0x02A0 => 0x02A7 (16 BITS WRITE - 16 BITS READ) (NOTE 1, 2)
-- SERIAL CONFIG FETCH COMMAND   = 0x02A8  (3 BITS WRITE - 3 BITS READ - TRIGGERS A READ CYCLE OF EEPROM)
-- TEMP / ADC CHANNEL PORT       = 0x02C0 => 0x02C7 (WRITE (NO SIGNIFICANT DATA - 12 BITS READ) (NOTE 1, 2)
-- TEMP / ADC CHANNEL MODE PORT  = 0x02C8 => 0x02CF (4 BITS WRITE - 4 BITS READ) (NOTE 1, 2)
-- SEQUENCER MPU PATTERN MEMORY  = 0x1000 => 0X1BFF (16 BITS WRITE - 16 BITS READ)
-- SEQUENCER MPU PROGRAM MEMORY  = 0x4000 => 0x43FF (16 BITS WRITE - 16 BITS READ)
-- NOTES:
-- 1. THESE FUNCTIONS ARE PHYSICALLY ETCHED INTO THE PIXEL FPGA LOGIC.
-- 2. NOT YET IMPLEMENETED OR TESTED.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE        AUTHOR  VERSION  MODIFICATIONS
-- 01/30/2003  PCM     0.1      ORIGINAL VERSION
-- ??/??/????  PCM     0.2 => 4.3 VARIOUS UNDOCUMENTED REVISIONS.
-- 02/14/2005  PCM     4.4      CORRECTED PATTERN GENERATOR ENABLE BIT FROM 15 TO BIT 7
--                              SINCE BIT 15 IS IN CONFLICT WITH SEQUENCER MASTER SYNC DELAY VALUE
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- READ       : Enables the tristate buffers to the pixel bus.
-- BOARD_SEL  : Global enable. Signifies that the board has been selected.  
-- CONTROL_REG: Output to seq interface to allow readback of content.
-- ADDRESS    : Address bus input for local decoding.
-- CONTROL_SEL: Output to the control reg in the sequencer interface module.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use work.FwVersionPkg.all;

entity Personality_ModuleV45 is
  port (SYSCLK       : in  std_logic;
        reset        : in  std_logic;
        WRITE        : in  std_logic;
        BOARD_SEL    : in  std_logic;
        LED_REG_SEL  : out std_logic;
        LED_DATA     : in  std_logic_vector(2 downto 0);
        TEMP_DATA_IN : in  std_logic_vector(9 downto 0);
        SN_DATA      : in  std_logic_vector(47 downto 0);
        ADDRESS      : in  std_logic_vector(15 downto 0);
        STATUS_REG   : in  std_logic_vector(15 downto 0);
        CLOCKEN_REG  : in  std_logic_vector(15 downto 0);
        SYNCDEL_REG  : in  std_logic_vector(7 downto 0);
        TST_CTR_REG  : in  std_logic_vector(31 downto 0);
        SEQ_CMD_REG  : in  std_logic_vector(15 downto 0);
        SEQ_INT_REG  : in  std_logic_vector(31 downto 0);
        SEQ_INT_CTR  : in  std_logic_vector(31 downto 0);
        SEQ_LPP_REG  : in  std_logic_vector(15 downto 0);
        SEQ_PAT_MEM  : in  std_logic_vector(15 downto 0);
        SEQ_PGM_MEM  : in  std_logic_vector(15 downto 0);
        ECHO_DATA_IN : in  std_logic_vector(31 downto 0);
        DATA_IN      : in  std_logic_vector(15 downto 0);
        DATA_OUT     : out std_logic_vector(31 downto 0);
        CONTROL_OUT  : out std_logic_vector(15 downto 0);
        WRITE_STB    : out std_logic;
        CLOCKEN_SEL  : out std_logic;
        SYNCDEL_SEL  : out std_logic;
        program_n    : out std_logic;
        SEQ_CMD_SEL  : out std_logic;
        SEQ_INT_SEL  : out std_logic;
        SEQ_LPP_SEL  : out std_logic;
        SEQ_PAT_SEL  : out std_logic;
        TST_CTR_REQ  : in  std_logic;
        TST_CTR_WRT  : out std_logic;
        TST_CTR_ENB  : out std_logic;
        TEMP_RD_STB  : out std_logic;
        SYNC_OUT     : out std_logic;
        READ         : in  std_logic;
        READ_STROBE  : in  std_logic;
        ECHO_REQ     : in  std_logic;
        CMD_OUT      : out std_logic;
        EXPOSE_START : out std_logic;
        FRAME_START  : out std_logic;
        MPU_EVENTS   : in  std_logic_vector(15 downto 0)
        );
end Personality_ModuleV45;


architecture Behavioral of Personality_ModuleV45 is

  component IdentityModule is
    port (board_ident_out : out std_logic_vector(15 downto 0);
          code_ident_out  : out std_logic_vector(15 downto 0)
          );
  end component;

  signal code_ident, board_ident              : std_logic_vector(15 downto 0);
  signal decode_vector                        : std_logic_vector(18 downto 0);
--  signal LocalData                            : std_logic_vector(31 downto 0);
  signal SignedTemp                           : std_logic_vector(31 downto 0);
  signal control_reg                          : std_logic_vector(15 downto 0);
  signal event_reg, event_hist_reg            : std_logic_vector(15 downto 0);
  signal LocalWrt                             : std_logic;
  signal PixCmdWrt, PixCmdWrtLock, PixCmdRead : std_logic;
  signal event_cs                             : std_logic;

-- Decoding Vectors
  signal CS16K_0, CS16K_1, CS16K_2, CS16K_3             : std_logic;
  signal CS4K_0, CS4K_1, CS4K_2, CS4K_3                 : std_logic;
  signal CS1K_0, CS1K_1, CS1K_2, CS1K_3                 : std_logic;
  signal CS256_0, CS256_1, CS256_2, CS256_3             : std_logic;
  signal CS64_0, CS64_1, CS64_2, CS64_3                 : std_logic;
  signal CS16_0, CS16_1, CS16_2, CS16_3                 : std_logic;
  signal CS_0, CS_1, CS_2, CS_3, CS_4, CS_5, CS_6, CS_7 : std_logic;
  signal CS_8, CS_9, CS_A, CS_B, CS_C, CS_D, CS_E, CS_F : std_logic;

-- Specific register addresses mapped into the bit vector.
  constant CONTROL_CS  : std_logic_vector(18 downto 0) := "0000000000000000001";
  constant STATUS_CS   : std_logic_vector(18 downto 0) := "0000000000000000010";
  constant BOARD_CS    : std_logic_vector(18 downto 0) := "0000000000000000100";
  constant CODE_CS     : std_logic_vector(18 downto 0) := "0000000000000001000";
  constant CLOCK_CS    : std_logic_vector(18 downto 0) := "0000000000000010000";
  constant SEQ_CTR_CS  : std_logic_vector(18 downto 0) := "0000000000000100000";
  constant SEQ_CMD_CS  : std_logic_vector(18 downto 0) := "0000000000001000000";
  constant SEQ_INT_CS  : std_logic_vector(18 downto 0) := "0000000000010000000";
  constant SEQ_LPP_CS  : std_logic_vector(18 downto 0) := "0000000000100000000";
  constant TST_CTR_CS  : std_logic_vector(18 downto 0) := "0000000001000000000";
  constant SEQ_PAT_CS  : std_logic_vector(18 downto 0) := "0000000010000000000";
  constant SEQ_PGM_CS  : std_logic_vector(18 downto 0) := "0000000100000000000";
  constant PIX_FPGA_CS : std_logic_vector(18 downto 0) := "0000001000000000000";
  constant LED_REG_CS  : std_logic_vector(18 downto 0) := "0000010000000000000";
  constant SN_DATA_CS  : std_logic_vector(18 downto 0) := "0000100000000000000";
  constant TEMPDAT_CS  : std_logic_vector(18 downto 0) := "0001000000000000000";
  constant SYNCDEL_CS  : std_logic_vector(18 downto 0) := "0010000000000000000";
  constant FW_VER_CS   : std_logic_vector(18 downto 0) := "0100000000000000000";
  constant FPGARLD_CS  : std_logic_vector(18 downto 0) := "1000000000000000000";
  
begin

------------------------------------------------------------------------
-- BOARD ADDRESS DECODE LOGIC
------------------------------------------------------------------------

-- Decode 16K word Blocks
  CS16K_0 <= (not ADDRESS(15)) and (not ADDRESS(14));
  CS16K_1 <= (not ADDRESS(15)) and ADDRESS(14);
--    CS16K_2 <= ADDRESS(15) and (not ADDRESS(14));
  CS16K_3 <= ADDRESS(15) and ADDRESS(14);

-- Decode 4K word Blocks
  CS4K_0 <= (not ADDRESS(13)) and (not ADDRESS(12));
  CS4K_1 <= (not ADDRESS(13)) and ADDRESS(12);
--    CS4K_2 <= ADDRESS(13) and (not ADDRESS(12));
  CS4K_3 <= ADDRESS(13) and ADDRESS(12);

-- Decode 1K word Blocks
  CS1K_0 <= (not ADDRESS(11)) and (not ADDRESS(10));
--    CS1K_1 <= (not ADDRESS(11)) and ADDRESS(10);
--    CS1K_2 <= ADDRESS(11) and (not ADDRESS(10));
  CS1K_3 <= ADDRESS(11) and ADDRESS(10);

-- Decode 256 word Blocks
  CS256_0 <= (not ADDRESS(9)) and (not ADDRESS(8));
  CS256_1 <= (not ADDRESS(9)) and ADDRESS(8);
  CS256_2 <= ADDRESS(9) and (not ADDRESS(8));
  CS256_3 <= ADDRESS(9) and ADDRESS(8);

-- Decode 64 word Blocks
  CS64_0 <= (not ADDRESS(7)) and (not ADDRESS(6));
--    CS64_1 <= (not ADDRESS(7)) and ADDRESS(6);
--    CS64_2 <= ADDRESS(7) and (not ADDRESS(6));
  CS64_3 <= ADDRESS(7) and ADDRESS(6);

  -- Decode 16 word Blocks
  CS16_0 <= (not ADDRESS(5)) and (not ADDRESS(4));
  CS16_1 <= (not ADDRESS(5)) and ADDRESS(4);
--    CS16_2 <= ADDRESS(5) and (not ADDRESS(4));
  CS16_3 <= ADDRESS(5) and ADDRESS(4);

-- Decode single word address lines
  CS_0 <= (not ADDRESS(3)) and (not ADDRESS(2)) and (not ADDRESS(1)) and (not ADDRESS(0));
  CS_1 <= (not ADDRESS(3)) and (not ADDRESS(2)) and (not ADDRESS(1)) and ADDRESS(0);
  CS_2 <= (not ADDRESS(3)) and (not ADDRESS(2)) and ADDRESS(1) and (not ADDRESS(0));
--    CS_3 <= (not ADDRESS(3)) and (not ADDRESS(2)) and ADDRESS(1) and ADDRESS(0);
--    CS_4 <= (not ADDRESS(3)) and ADDRESS(2) and (not ADDRESS(1)) and (not ADDRESS(0));
--    CS_5 <= (not ADDRESS(3)) and ADDRESS(2) and (not ADDRESS(1)) and ADDRESS(0);
--    CS_6 <= (not ADDRESS(3)) and ADDRESS(2) and ADDRESS(1) and (not ADDRESS(0));
--    CS_7 <= (not ADDRESS(3)) and ADDRESS(2) and ADDRESS(1) and ADDRESS(0);
--    CS_8 <= ADDRESS(3) and (not ADDRESS(2)) and (not ADDRESS(1)) and (not ADDRESS(0));
  CS_9 <= ADDRESS(3) and (not ADDRESS(2)) and (not ADDRESS(1)) and ADDRESS(0);
  CS_A <= ADDRESS(3) and (not ADDRESS(2)) and ADDRESS(1) and (not ADDRESS(0));
  CS_B <= ADDRESS(3) and (not ADDRESS(2)) and ADDRESS(1) and ADDRESS(0);
  CS_C <= ADDRESS(3) and ADDRESS(2) and (not ADDRESS(1)) and (not ADDRESS(0));
  CS_D <= ADDRESS(3) and ADDRESS(2) and (not ADDRESS(1)) and ADDRESS(0);
  CS_E <= ADDRESS(3) and ADDRESS(2) and ADDRESS(1) and (not ADDRESS(0));
  CS_F <= ADDRESS(3) and ADDRESS(2) and ADDRESS(1) and ADDRESS(0);


-- Specific decode values for interface registers


-- Reprogram fpga, address 0x0666 - read / write
  decode_vector(18) <= '1' when (BOARD_SEL = '1' and ADDRESS = x"0666")
                       else '0';
  -- when writing any value to register 666 generate a pulse after 100ms
  program_n_delay: process (SYSCLK, reset)
    variable tickCount : integer range 0 to 65535 := 0;
    variable msCount : integer range 0 to 127 := 0;
    variable run : std_logic;
  begin  -- process program_n_delay
    if reset = '1' then                 -- asynchronous reset (active high)
      program_n <= '1';
      run := '0';
      tickCount := 0;
      msCount := 0;
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if (decode_vector(18) and write and (not run)) = '1' then
        run := '1';
        tickCount := 0;
        msCount := 0;
        program_n <= '1';
      elsif run = '1' then
        if tickCount = 40000 then       -- 40000 ticks = 1 ms
          tickCount := 0;
          if msCount < 100 then         -- delay of 100ms
            msCount := msCount + 1;
          else
            program_n <= '0';
            msCount := 0;
            run := '0';
          end if;
        else
          tickCount := tickCount + 1;
        end if;
      else
        program_n <= '1';
        run := '0';
      end if;
    end if;
  end process program_n_delay;
--  program_n <= not(decode_vector(18) and WRITE);

-- Firmware version register at 0x0301 - read
  decode_vector(17) <= '1' when (BOARD_SEL = '1' and ADDRESS = x"0301")
                       else '0';

-- Sync delay register at 0x0300 - read / write
  decode_vector(16) <= '1' when (BOARD_SEL = '1' and ADDRESS = x"0300")
                       else '0';
  SYNCDEL_SEL <= decode_vector(16) and WRITE;

-- Event register at 0xfff0 - provides strobes for common control functions
  event_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_0;

-- LED indicator register at address 0xfff9 - read / write
  decode_vector(13) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_9;
  LED_REG_SEL       <= decode_vector(13) and WRITE;

-- Circuit board serial number at address 0xfffa - read only
  decode_vector(14) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_A;

-- Cicuit board temperature at address 0xfffb - read only (write triggers read of sensor) 
  decode_vector(15) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_B;
  TEMP_RD_STB       <= decode_vector(15) and WRITE;

-- Control Register select at 0xfffc
  decode_vector(0) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_C;

-- Status Register select at 0xfffd
  decode_vector(1) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_D;

-- Board id Register select at 0xfffe
  decode_vector(2) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_E;

-- Code id Register select at 0xffff
  decode_vector(3) <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_F;

-- Board specific values for logic implementation

-- Sequencer required integration time register selected at 0x0000 (32 bit write space)
  decode_vector(7) <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_0;
  SEQ_INT_SEL      <= decode_vector(7) and WRITE;

-- Test counter register selected at 0x0001 (32 bit write space)
  decode_vector(9) <= (BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_1) or TST_CTR_REQ;
  TST_CTR_WRT      <= decode_vector(9)and WRITE;

-- Clock enable register selected at 0x0100
  decode_vector(4) <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_0;
  CLOCKEN_SEL      <= decode_vector(4) and WRITE;

-- Sequencer actual integration  time register selected at 0x0101
  decode_vector(5) <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_1;

-- Sequencer Command register selected at 0x0102
  decode_vector(6) <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_2;
  SEQ_CMD_SEL      <= decode_vector(6) and WRITE;

-- Sequencer Loop registers selected from 0x110 to 0x11F
  decode_vector(8) <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1;
  SEQ_LPP_SEL      <= decode_vector(8) and WRITE;

-- Sequencer Pattern memory selected at 0x1000 to 0x1bff (Actual memory is 3 K words of 16 bit 0x1000 => 0x1BFF)
  decode_vector(10) <= BOARD_SEL and CS16K_0 and CS4K_1 and not CS1K_3;
  SEQ_PAT_SEL       <= decode_vector(10) and WRITE;

-- Sequencer Program memory selected at 0x4000 to 0x43ff (Actual memory is 1 K words of 16 bit 0x2000 => 0x23FF)
  decode_vector(11) <= BOARD_SEL and CS16K_1 and CS4K_0 and CS1K_0;
--    SEQ_PGM_SEL <= decode_vector(11);

-- Pixel Fpga Command segment at 0x0200 => 0x2FF
  decode_vector(12) <= (BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_2) or PixCmdWrt;

------------------------------------------------------------------------
-- IDENTITY CONSTANTS
-- Map the identity module into this code module signals
------------------------------------------------------------------------

  IDENT : IdentityModule port map (board_ident_out => board_ident, code_ident_out => code_ident);

------------------------------------------------------------------------
-- The control register
------------------------------------------------------------------------
  Cntrl_reg : process (SYSCLK, reset)
  begin  -- process Cntrl_reg
    if reset = '1' then                 -- asynchronous reset (active high)
      event_reg   <= (others => '0');
      control_reg <= (others => '0');
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if WRITE = '1' then
        if decode_vector(0) = '1' then
          control_reg <= DATA_IN(15 downto 0);
        elsif event_cs = '1' then
          event_reg <= DATA_IN(15 downto 0);
        end if;
      else
        event_reg <= (others => '0');
      end if;
    end if;
  end process Cntrl_reg;

  CONTROL_OUT  <= control_reg;
  TST_CTR_ENB  <= control_reg(7);
  EXPOSE_START <= event_reg(15) or MPU_EVENTS(15);
  FRAME_START  <= event_reg(0) or MPU_EVENTS(0);

  EventHistoryProc : process (SYSCLK, reset)
  begin  -- process EventHistoryProc
    if reset = '1' then                 -- asynchronous reset (active high)
      event_hist_reg <= (others => '0');
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if event_reg(15) = '1' then
        event_hist_reg <= (others => '0');
      else
        event_hist_reg <= event_hist_reg or event_reg or MPU_EVENTS;
      end if;
    end if;
  end process EventHistoryProc;


------------------------------------------------------------------------
-- Sign extension for those data which return a signed result
------------------------------------------------------------------------

  SignedTemp <= (x"00000" & "00" & TEMP_DATA_IN) when (TEMP_DATA_IN(9) = '0') else
                (x"FFFFF" & "11" & TEMP_DATA_IN);

------------------------------------------------------------------------
-- DATA MULTIPLEXOR
-- If Status register is required, write in the port definition and
-- Remove the commented line to enable the status to be read accross
-- the bus. Add any data ports you require to this mux with relevent 
-- 'chip selects' derived from the address decode logic.
------------------------------------------------------------------------
  DataMuxMux : process (SYSCLK, reset)
  begin  -- process DataMuxMux
    if reset = '1' then                 -- asynchronous reset (active high)
      DATA_OUT <= x"12345678";
    elsif SYSCLK'event and SYSCLK = '1' then               -- rising clock edge
      if (ECHO_REQ = '1') then
        DATA_OUT <= ECHO_DATA_IN;
      else
        case decode_vector is
          when CONTROL_CS =>
            DATA_OUT <= x"0000" & control_reg;
          when STATUS_CS =>
            DATA_OUT <= (x"0000" & STATUS_REG(15 downto 0));
          when BOARD_CS =>
            DATA_OUT <= (x"0000" & board_ident(15 downto 0));
          when CODE_CS =>
            DATA_OUT <= (x"0000" & code_ident(15 downto 0));
          when CLOCK_CS =>
            DATA_OUT <= (x"0000" & CLOCKEN_REG(15 downto 0));
          when SEQ_CTR_CS =>
            DATA_OUT <= SEQ_INT_CTR;
          when SEQ_CMD_CS =>
            DATA_OUT <= (x"0000" & SEQ_CMD_REG(15 downto 0));
          when SEQ_INT_CS =>
            DATA_OUT <= SEQ_INT_REG;
          when SEQ_LPP_CS =>
            DATA_OUT <= (x"0000" & SEQ_LPP_REG(15 downto 0));
          when TST_CTR_CS =>
            DATA_OUT <= TST_CTR_REG;
          when SEQ_PAT_CS =>
            DATA_OUT <= (x"0000" & SEQ_PAT_MEM(15 downto 0));
          when SEQ_PGM_CS =>
            DATA_OUT <= (x"0000" & SEQ_PGM_MEM(15 downto 0));
          when PIX_FPGA_CS =>
            DATA_OUT <= ADDRESS & DATA_IN;
          when LED_REG_CS =>
            DATA_OUT <= (x"0000000" & '0' & LED_DATA);
          when SN_DATA_CS =>
            DATA_OUT <= SN_DATA(31 downto 0);
          when TEMPDAT_CS =>
            DATA_OUT <= (x"00000" & "00" & TEMP_DATA_IN);  -- SignedTemp;
          when SYNCDEL_CS =>
            DATA_OUT <= x"000000" & SYNCDEL_REG;
          when FW_VER_CS =>
            DATA_OUT <= x"0000"& FW_VERSION;
          when FPGARLD_CS =>
            DATA_OUT <= x"000BEA57";
          when others =>
            DATA_OUT <= x"0DEFA017";
        end case;
      end if;
    end if;
  end process DataMuxMux;

------------------------------------------------------------------------
-- FPGA INTERFACE WRITE STROBE DEVELOPMENT
------------------------------------------------------------------------
  LocalWrt <= (READ_STROBE and BOARD_SEL) or ECHO_REQ or TST_CTR_REQ or PixCmdWrt;

  DataMuxWriteStrobe : process (SYSCLK, reset)
  begin  -- process DataMuxWriteStrobe
    if reset = '1' then                 -- asynchronous reset (active high)
      WRITE_STB <= '0';
      SYNC_OUT  <= '0';                 -- Not sure if it is active low or high
      CMD_OUT   <= '0';                 -- same applies
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if (LocalWrt = '1') then
        WRITE_STB <= '1';
        if (TST_CTR_REQ = '1') then
          SYNC_OUT <= '0';  -- Sync should be false for any synthetic pixel data sent to pan.
          CMD_OUT  <= '0';
        elsif (PixCmdWrt = '1') then
          SYNC_OUT <= PixCmdRead;  -- Sync used to carry the command type if command being sent to Pix Fpga.
          CMD_OUT  <= '1';
        else
          SYNC_OUT <= '1';  -- Sync true for anything else sent to the pan.
          CMD_OUT  <= '0';
        end if;
      else
        WRITE_STB <= '0';
      end if;
    end if;
  end process DataMuxWriteStrobe;

  PixCmdWrtStrobe : process (SYSCLK, reset)
  begin  -- process PixCmdWrtStrobe
    if reset = '1' then                 -- asynchronous reset (active high)
      PixCmdWrt     <= '0';
      PixCmdWrtLock <= '0';
      PixCmdRead    <= '0';             -- No idea what we should put
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if (Decode_vector(12) = '1') then  -- command passing over to the pix fpga.
        if (PixCmdWrtLock = '0') then
          PixCmdRead    <= READ;        -- Capture the command type
          PixCmdWrt     <= '1';
          PixCmdWrtLock <= '1';
        else
          PixCmdWrt <= '0';
        end if;
      else
        PixCmdWrtLock <= '0';
        PixCmdWrt     <= '0';
      end if;
    end if;
  end process PixCmdWrtStrobe;

end Behavioral;
