------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   DHE SYNCHRONIZATION LOGIC
-- VERSION      :   4.3 
------------------------------------------------------------------------
-- FILE NAME    :   SYNCLOGIC.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   12/06/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES THE TIMING TO ENSURE THAT MULTIPLE DHE UNITS ARE SYNCHRONIZED
-- IN THE EXECUTION OF THE SEQUENCER MPU PROGRAM. 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- This is a non-retriggerable one shot that provides the following waveform
-- (Several events occur here) :
--                     /-----------\          /--\
-- SYNC_EFR     ______/             \________/    \___
--                                  |
--                      /-----------|---------------------------\
-- EXT_SYNC_OUT _______/            |                            \_________
--                     |            |------------ DELAY_MS ------|
--                     |                                         |
--                     |<= Seq clock sync'ed on rising edge      |
--                                                               |
--           Typical [#WAIT JCB #SYNC #WAIT] instruction falls   |
--           out of loop on trailing edge   ====================>|
--
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE      AUTHOR  VERSION  MODIFICATIONS
-- 05/14/2004  PCM      4.2   ORIGINAL VERSION
-- 12/06/2004  PCM      4.3   REPORTED MODULE TO USE ALL OF THE DATA_CTL_REG SINCE
--                            SOMEONE HAD TO DO IT !
-- 10 Dec 08   JTO            add sync reset
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK       : 40 MHz system clock
-- SEQ_RUNNING  : Resets the logic to known state when not true.
-- MASTER       : flag to indicate whether we are master or slave dhe.
-- EXT_SYNC_OUT : SYNC bit out to next DHE
-- SYNC_EFR     : SYNC request from our sequencer if we are master
-- EXT_SYNC_IN  : SYNC bit input from previous dhe in daisy chain
-- SYNC_FLAG    : flag that indicates sync status to the sequencer
-- DELAY_MS     : Delay (in ms.) from when SYNC_EFR changes state to when EXT_SYNC_OUT goes true.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SyncLogic is
    Port ( SYSCLK       : in  std_logic;
           reset        : in std_logic;
         DATA_CTL_REG : in  std_logic_vector(15 downto 0);
         EXT_SYNC_OUT : out std_logic;
         SYNC_EFR     : in  std_logic;
         EXT_SYNC_IN  : in  std_logic;
         SYNC_FLAG    : out std_logic
         );
end SyncLogic;

architecture Behavioral of SyncLogic is
  constant ONE_MILLISEC : std_logic_vector(15 downto 0) := x"9C40";
  signal   MSecCntr     : std_logic_vector(15 downto 0);
  signal   DelayCnt     : std_logic_vector(7 downto 0);
  signal   DlyCntr      : std_logic_vector(7 downto 0);
  signal   MSecFlag     : std_logic;
  signal   Trigger      : std_logic;
  signal   TrigArm      : std_logic;
  signal   SyncBit      : std_logic;
  signal   SEQ_RUNNING  : std_logic;
  signal   MASTER       : std_logic;
  signal   DELAY_MS     : std_logic_vector(7 downto 0);

begin

  SEQ_RUNNING  <= DATA_CTL_REG(0);
  MASTER       <= DATA_CTL_REG(4);
  DELAY_MS     <= DATA_CTL_REG(15 downto 8);
  EXT_SYNC_OUT <= (SyncBit or TrigArm) when (MASTER = '1') else 'Z';
  SYNC_FLAG    <= EXT_SYNC_IN;

------------------------------------------------------------------------
-- NEGATIVE EDGE TRIGGERED ONE SHOT
------------------------------------------------------------------------

--    NegEdgeTrigProc : process(SYSCLK)
--    begin
--        if (rising_edge(SYSCLK)) then
--            if (SEQ_RUNNING = '1' and SyncBit = '0') then
--                if (SYNC_EFR = '1') then
--                    TrigArm <= '1';
--                elsif (TrigArm = '1') then
--                    Trigger <= '1';
--                    DelayCnt <= DELAY_MS;           -- Capture the required delay time 
--                end if;
--            else
--                TrigArm <= '0';
--                Trigger <= '0';
--            end if;
--        end if;
--    end process;

  NegEdgeTrigProc : process(SYSCLK)
  begin
        if rising_edge(SYSCLK) then

            if (reset='1') then
                TrigArm <= '0';
                Trigger <= '0';
                DelayCnt <= (others=>'0');
            else

      if (SEQ_RUNNING = '1' and SyncBit = '0') then
        if (SYNC_EFR = '1') then
          TrigArm <= '1';
        elsif (TrigArm = '1') then
          Trigger  <= '1';
          DelayCnt <= DELAY_MS;         -- Capture the required delay time 
        end if;
      else
        TrigArm <= '0';
        Trigger <= '0';
      end if;

               end if;
    end if;
  end process;

------------------------------------------------------------------------
-- DELAY TIMER
------------------------------------------------------------------------
--    DlySyncProc : process(SYSCLK)
--    begin
--        if (rising_edge(SYSCLK)) then
--            if (SEQ_RUNNING = '1') then
--                if (DlyCntr < DelayCnt) then
--                    if (MSecFlag = '1') then
--                        DlyCntr <= DlyCntr + 1;
--                    end if;
--                elsif (SyncBit = '1') then
--                    SyncBit <= '0';
--                elsif (Trigger = '1') then
--                    DlyCntr <= (others => '0');
--                    SyncBit <= '1';
--                end if;
--            else
--                DlyCntr <= (others => '1');
--                SyncBit <= '0';
--            end if;
--        end if;
--    end process;

  DlySyncProc : process(SYSCLK)
  begin
        if rising_edge(SYSCLK) then
            if (reset='1') then
                SyncBit <= '0';
                DlyCntr <= (others=>'0');
            else
      if (SEQ_RUNNING = '1') then
        if (DlyCntr < DelayCnt) then
          if (MSecFlag = '1') then
            DlyCntr <= DlyCntr + 1;
          end if;
        elsif (SyncBit = '1') then
          SyncBit <= '0';
        elsif (Trigger = '1') then
          DlyCntr <= (others => '0');
          SyncBit <= '1';
        end if;
      else
        DlyCntr <= (others => '1');
        SyncBit <= '0';
      end if;
    end if;
        end if;
  end process;

------------------------------------------------------------------------
-- MILLISECOND TIMER
------------------------------------------------------------------------

--    MilliSecTimerProc : process (SYSCLK)
--    begin
--        if (rising_edge(SYSCLK)) then
--            if (TrigArm = '1') then             -- Hold off to synchronize timer to edge.
--                MSecCntr <= (others => '0');
--                MSecFlag <= '0';
--            elsif (MSecCntr < ONE_MILLISEC) then
--                MSecCntr <= MSecCntr + 1;
--                MSecFlag <= '0';
--            else
--                MSecCntr <= (others => '0');
--                MSecFlag <= '1';
--            end if;
--        end if;
--    end process;

  MilliSecTimerProc : process (SYSCLK)
  begin
        if rising_edge(SYSCLK) then
            if (reset='1') then
                MSecFlag <= '0';
                MSecCntr <= (others=>'0');
            else
      if (TrigArm = '1') then  -- Hold off to synchronize timer to edge.
        MSecCntr <= (others => '0');
        MSecFlag <= '0';
      elsif (MSecCntr < ONE_MILLISEC) then
        MSecCntr <= MSecCntr + 1;
        MSecFlag <= '0';
      else
        MSecCntr <= (others => '0');
        MSecFlag <= '1';
      end if;
    end if;
        end if;
  end process;

end Behavioral;
