------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   TEMPERATURE SENSOR READ LOGIC
-- VERSION      :   4.2 
------------------------------------------------------------------------
-- FILE NAME    :   TEMPSENSOR.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/19/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- INTERFACES A AD7814 3 WIRE TEMP SENSOR.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	     AUTHOR  VERSION  MODIFICATIONS
-- 05/19/2004  PCM      4.2   ORIGINAL VERSION
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK    : SYSTEM CLOCK
-- READ_TEMP : TRIGGERS A READ FROM THE TEMPERATURE SENSOR
-- TEMP_DATA : TEMPERATURE DATA OUT TO MUX
-- TEMP_DIN  : SERIAL DATA INTO SENSOR
-- TEMP_DOUT : SERIAL DATA OUT FROM SENSOR
-- TEMP_SCLK : SERIAL SENSOR DATA CLOCK
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TempSensor is
    Port ( SYSCLK    : in  std_logic;
           READ_TEMP : in  std_logic;
           TEMP_DATA : out std_logic_vector(9 downto 0);
           TEMP_DIN  : out std_logic;
           TEMP_DOUT : in  std_logic;
           TEMP_SCLK : out std_logic
         );
end TempSensor;

architecture Behavioral of TempSensor is
-- Serial clock generator signals
    constant SERCLKRATE : std_logic_vector(2 downto 0) := "010";        -- Clock period is currently set to 3 clk periods i.e 75ns per phase
    constant SERFRAME   : std_logic_vector(4 downto 0) := "10000";      -- Number of bits in a serial frame
    signal SerClkCntr   : std_logic_vector(2 downto 0);
    signal SerBitCntr   : std_logic_vector(4 downto 0);
    signal SerClk       : std_logic;                                    -- Actual serial clock signal
    signal SerClkEn     : std_logic;                                    -- Enables serial clock to run
-- Serial data read signals
    signal SerDataIn    : std_logic_vector(15 downto 0);                -- Serial to parallel converter
    signal SerDataOut   : std_logic_vector(15 downto 0);                -- Parallel to serial converter
    signal DataInLatch  : std_logic_vector(9 downto 0);                 -- Latched version of SerDataIn after acquisition.
-- Serial control signals
    constant TEMPMEAS   : std_logic_vector(9 downto 0) := "1111101000"; -- 1000 clock cycles is 25us. 
    type TempStateType is (TempIdle, TempWakeUp, TempMeasure, TempRead);
    signal TempState    : TempStateType;
    signal TempTimer    : std_logic_vector(9 downto 0);                 -- Timer counter to wait for sensor response
    signal SerPwrDown   : std_logic;                                    -- Controls whether we power down the sensor or not.

    signal serclk_reg   : std_logic;

begin

------------------------------------------------------------------------
-- Temperature acquisition control.
-- Must wake sensor up, wait 25us for an update, read data and shut it down again.
------------------------------------------------------------------------
    TempCtlProc : process(SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (READ_TEMP = '1') then
                TempState <= TempWakeUp;                    -- Trigger the FSM to wake up the sensor.
            else
                case TempState is
                    when TempIdle =>
                        SerPwrDown <= '0';                  -- Prepare to enable the temp sensor when we run.
                        TempState <= TempIdle;              -- and hang around here 'till we're next commanded ..... 
                    when TempWakeUp =>                       
                        if (SerBitCntr < SERFRAME) then
                            SerClkEn <= '1';                -- Enable the serial processes to run
                            TempState <= TempWakeUp;        -- while the bit counter less than one frame
                        else
                            SerClkEn <= '0';
                            TempTimer <= (others => '0');
                            TempState <= TempMeasure;
                        end if;
                    when TempMeasure =>
                        if (TempTimer < TEMPMEAS) then      -- Hang around for 25us for temp sensor to respond
                            TempTimer <= TempTimer + 1;
                            TempState <= TempMeasure;
                        else
                            TempState <= TempRead;
                            SerPwrDown <= '1';              -- Set up to power down the sensor in the next read / write process.
                        end if;
                    when TempRead =>
                        if (SerBitCntr < SERFRAME) then     -- Again, enable the serial processes and read back 
                            SerClkEn <= '1';                -- the temperature while shutting down the sensor.
                            TempState <= TempRead;
                        else
                            SerClkEn <= '0';
                            TempState <= TempIdle;
                        end if;
                    when Others =>
                        TempState <= TempIdle;
                end case;
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- Serial clock generator ... Clock period must be > 100ns.
------------------------------------------------------------------------
    SerClkProc : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (SerClkEn = '1') then
                if (SerClkCntr < SERCLKRATE) then
                    SerClkCntr <= SerClkCntr + 1;
                else
                    SerClkCntr <= (others => '0');
                    SerClk <= not SerClk;
                end if;
            else
                SerClkCntr <= (others => '1');          -- Assures that after one sysclk after SerClkEn goes true, SerClk goes low.
                Serclk <= '1';                          -- Quiescent state of clock is high
            end if;
        end if;
    end process;

    TEMP_SCLK <= SerClk;

    -- detect the rising edge of the slower serclk using the faster
    -- sysclk to elimnate a clock domain.

    edge_detect : process (SYSCLK)
    begin
        if rising_edge(SYSCLK) then
            serclk_reg <= serclk;
        end if;
    end process edge_detect;


--    SerBitCntrProc : process(SerClk, SerClkEn)
--    begin
--        if (SerClkEn = '1') then
--            if (rising_edge(SerClk)) then
--                SerBitCntr <= SerBitCntr + 1;           -- Increase the bit counter.
--            end if;
--        else
--            SerBitCntr <= (others => '0');              -- Start the process on bit zero, count up to 16 which signals end of serial frame. 
--        end if;  
--    end process;

    SerBitCntrProc : process(sysclk)
    begin
        if rising_edge(sysclk) then
        if (SerClkEn = '1') then
                if (serclk='1' and serclk_reg='0') then  -- rising edge serclk
                SerBitCntr <= SerBitCntr + 1;           -- Increase the bit counter.
            end if;
        else
            SerBitCntr <= (others => '0');              -- Start the process on bit zero, count up to 16 which signals end of serial frame. 
        end if;  
        end if;
    end process;

------------------------------------------------------------------------
-- Serial to parallel converter. 16 bits read
------------------------------------------------------------------------
--    SerReadProc : process(SerClk, SerClkEn)
--    begin
--        if (SerClkEn = '1') then
--            if (rising_edge(SerClk)) then
--                SerDataIn(15 downto 0) <= SerDataIn(14 downto 0) & TEMP_DOUT; -- Left shift - MSB in first.
--            end if;
--        else
--            DataInLatch <= SerDataIn(14 downto 5);
--        end if;    
--    end process;

    SerReadProc : process(sysclk)
    begin
        if rising_edge(sysclk) then
        if (SerClkEn = '1') then
                if (serclk='1' and serclk_reg='0') then -- rising_edge of serclk
                SerDataIn(15 downto 0) <= SerDataIn(14 downto 0) & TEMP_DOUT; -- Left shift - MSB in first.
            end if;
        else
            DataInLatch <= SerDataIn(14 downto 5);
        end if;    
        end if;
    end process;

    TEMP_DATA <= DataInLatch;

------------------------------------------------------------------------
-- Parallel to serial converter. 16 bits sent
------------------------------------------------------------------------
--    SerWrtProc : process(SerClk, SerClkEn, SerPwrDown)
--    begin
--        if (SerClkEn = '0') then
--            SerDataOut <= "00" & SerPwrDown & "0000000000000";
--        elsif (falling_edge(SerClk)) then
--            TEMP_DIN <= SerDataOut(15);
--            SerDataOut <= SerDataOut(14 downto 0) & '0';    -- Left shift - MSB out first.
--        end if;      
--    end process;

    SerWrtProc : process(sysclk)
    begin
        if rising_edge(sysclk) then
        if (SerClkEn = '0') then
            SerDataOut <= "00" & SerPwrDown & "0000000000000";
            elsif (serclk='0' and serclk_reg='1') then -- serclk falling edge
            TEMP_DIN <= SerDataOut(15);
            SerDataOut <= SerDataOut(14 downto 0) & '0';    -- Left shift - MSB out first.
        end if;      
        end if;      
    end process;

end Behavioral;
