-------------------------------------------------------------------------------
-- Title      : Sync Delay Chip interface
-- Project    : 
-------------------------------------------------------------------------------
-- File       : syncDelay.vhd
-- Author     :   <otger@PWOTGER>
-- Company    : 
-- Created    : 2009-01-19
-- Last update: 2009-01-21
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2009 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2009-01-19  1.0      otger	Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity syncDelay is
  
  port (
    sysclk      : in  std_logic;
    reset       : in  std_logic;
    write       : in  std_logic;
    delayDatIn  : in  std_logic_vector(7 downto 0);
    delayDatOut : out std_logic_vector(7 downto 0);
    delChipClk  : out std_logic;
    delChipD    : out std_logic;
    delChipE    : out std_logic);

end syncDelay;

architecture behavioral of syncDelay is

  constant clkDiv : integer := 0;       -- clock divider for chip serial
                                        -- interface 

  type syncStateTypes is(
    idle,
    eUp,
    setD,
    waitCDown,
    cDown,
    waitCup,
    cUp,
    waitSetD,
    waitEdown,
    eDown);
  signal syncCurState, syncNextState : syncStateTypes;

  signal syncDelay : std_logic_vector(7 downto 0);
  signal counter : integer range 0 to 7;
  signal countCUp : integer range 0 to 3;
  signal slowClk : std_logic;
  signal clockDivCounter : integer range 0 to clkDiv;
  signal program : std_logic;
  
begin  -- behavioral

  -- purpose: Register to hold syncDelay value
  -- type   : sequential
  -- inputs : sysclk, reset
  -- outputs: syncDel
  syncDelReg: process (sysclk, reset)
  begin  -- process syncDelReg
    if reset = '1' then                 -- asynchronous reset (active high)
      syncDelay <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if write = '1' then
        syncDelay <= delayDatIn;
      end if;
    end if;
  end process syncDelReg;

  -- Output delay value so it can be read
  delayDatOut <= syncDelay;


  -- purpose: generate a program signal to triger programming of chip
  -- type   : sequential
  -- inputs : sysclk, reset, write, syncCurState
  -- outputs: program
  programProc: process (sysclk, reset)
  begin  -- process programProc
    if reset = '1' then                 -- asynchronous reset (active high)
      program <= '0';
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if syncCurState = idle and write = '1' then
        program <= '1';
      elsif syncCurState = eUp then
        program <= '0';
      end if;
    end if;
  end process programProc;

  -- purpose: generate a slower clock
  -- type   : sequential
  -- inputs : sysclk, reset
  -- outputs: 
  slowClkProc: process (sysclk, reset)
--    variable clockDivCounter : integer range 0 to clkDiv;
  begin  -- process slowClkProc
    if reset = '1' then                 -- asynchronous reset (active low)
      slowClk <= '0';
      clockDivCounter <=  clkDiv;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clockDivCounter = 0 then
        slowClk <= not slowClk;
        clockDivCounter <=  clkDiv;
      else
        clockDivCounter <= clockDivCounter - 1;
      end if;
    end if;
  end process slowClkProc;
  
  -- purpose: Assign next state to current state
  -- type   : sequential
  -- inputs : slowClk, reset, syncNextState
  -- outputs: syncCurState
  curStateProc: process (slowClk, reset)
  begin  -- process curStateProc
    if reset = '1' then                 -- asynchronous reset (active high)
      syncCurState <= idle;
    elsif slowClk'event and slowClk = '1' then  -- rising clock edge
      syncCurState <= syncNextState;
    end if;
  end process curStateProc;

  -- Generate Next state

  syncNextState <= eUp when syncCurState = idle and program = '1' else
                   idle when syncCurState = idle else
                   setD when syncCurState = eUp else
                   waitCDown when syncCurState = setD else
                   cDown when syncCurState = waitCDown else
                   waitCup when syncCurState = cDown else
                   waitCup when syncCurState = waitCup and countCUp > 0 else
                   cUp when syncCurState = waitCup else
                   waitSetD when syncCurState = cUp and counter > 0 else
                   setD when syncCurState = waitSetD else
                   waitEdown when syncCurState = cUp else
                   eDown when syncCurState = waitEdown else
                   idle;

  -- purpose: generate a counter
  -- type   : sequential
  -- inputs : slowClk, reset, syncNextState
  -- outputs: counter
  counterProc: process (slowClk, reset)
  begin  -- process counterProc
    if reset = '1' then                 -- asynchronous reset (active low)
      counter <= 7;
    elsif slowClk'event and slowClk = '1' then  -- rising clock edge
      if syncNextState = idle then
        counter <= 7;
      elsif syncNextState = waitSetD then
        counter <= counter - 1;
      end if;
    end if;
  end process counterProc;

  -- purpose: generate a counter
  -- type   : sequential
  -- inputs : slowClk, reset, syncNextState
  -- outputs: counter
  countCupProc: process (slowClk, reset)
  begin  -- process countCupProc
    if reset = '1' then                 -- asynchronous reset (active low)
      countCUp <= 3;
    elsif slowClk'event and slowClk = '1' then  -- rising clock edge
      if syncNextState = idle then
        countCUp <= 3;
      elsif syncNextState = cDown then
        countCUp <= 3;
      elsif syncNextState = waitCup then
        countCUp <= countCUp - 1;
      end if;
    end if;
  end process countCupProc;

  -- purpose: generate Enable
  -- type   : sequential
  -- inputs : slowClk, reset, syncNextState
  -- outputs: delChipE
  delChipEProc: process (slowClk, reset)
  begin  -- process delChipEProc
    if reset = '1' then                 -- asynchronous reset (active high)
      delChipE <= '0';
    elsif slowClk'event and slowClk = '1' then  -- rising clock edge
      if syncNextState = eUp then
        delChipE <= '1';
      elsif syncCurState = eDown then
        delChipE <= '0';
      end if;
    end if;
  end process delChipEProc;

  -- purpose: generate data to write to delay chip
  -- type   : sequential
  -- inputs : slowClk, reset
  -- outputs: 
  delChipDProc: process (slowClk, reset)
  begin  -- process delChipDProc
    if reset = '1' then                 -- asynchronous reset (active high)
      delChipD <= '0';
    elsif slowClk'event and slowClk = '1' then  -- rising clock edge
      if syncNextState = setD then
        delChipD <= syncDelay(counter);
      end if;
    end if;
  end process delChipDProc;

  -- purpose: generate delay chip serial interface clock
  -- type   : sequential
  -- inputs : slowClk, reset
  -- outputs: 
  delChipCProc: process (slowClk, reset)
  begin  -- process delChipCProc
    if reset = '1' then                 -- asynchronous reset (active low)
      delChipClk <= '1';
    elsif slowClk'event and slowClk = '1' then  -- rising clock edge
      if syncNextState = idle then
        delChipClk <= '1';
      elsif syncNextState = cDown then
        delChipClk <= '0';
      elsif syncNextState = cUp then
        delChipClk <= '1';
      end if;
    end if;
  end process delChipCProc;
  

end behavioral;
