-- simple model of the CCDACQ12 board
-- uses real timing and does generate an ACK
-- all registers read back the same thing.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.image_pkg.all;                 -- ben cohen's image utilities

entity ccd12 is
  port(
    clock        : in  std_logic;
    board_slct_n : in  std_logic;
    seq_mode     : in  std_logic_vector(1 downto 0);
    dev_addr     : in  std_logic_vector(5 downto 0);
    seq_data     : in  std_logic_vector(31 downto 0);
    pix_data     : out std_logic_vector(47 downto 0);
    ack_n        : out std_logic
    );
end ccd12;

architecture ccd12_arch of ccd12 is

  signal pix_temp : std_logic_vector(47 downto 0);

begin

-- model the CCD board in slot 6
-- this is a simple model that responds to read/write requests
-- when reading data from the CCD12 board the timing is fairly accurate
-- acks are also generated (MCB may not care).

  ccd_write : process(clock)
  begin
    if rising_edge(clock) then
      if (board_slct_n = '0') then
        if (seq_data = "00" and dev_addr(5 downto 0)="000000") then  -- reset
          report "CCD12: soft reset by MCB" severity warning;
        elsif (seq_data = "01") then    -- read
          report "CCD12: MCB is reading address 0x" & HexImage(seq_data(31 downto 16)) severity warning;
        elsif (seq_data = "10") then    -- 16 bit write
          report "CCD12: MCB is writing 0x" & HexImage(seq_data(31 downto 16)) & " to address 0x" & HexImage(seq_data(15 downto 0)) severity warning;
        else                            -- 32 bit write
          report "CCD12: MCB is writing 0x" & HexImage(seq_data(31 downto 0)) & " to address 0x" & HexImage(dev_addr(3 downto 0)) severity warning;
        end if;
      end if;
    end if;
  end process ccd_write;

-- reading from the CCD12 board is totally async and depends only on the 
-- seq_mode and board_slct_n lines.  Always reads the same data.

  pix_temp <= X"12345678ABCD" when (board_slct_n = '0' and seq_mode = "01") else (others =>'H');
  pix_data <= pix_temp after 30 ns;

-- generate the optional ACK back to the MCB

  ccd_ack : process
  begin
    ack_n <= '1';
    wait until (rising_edge(clock) and board_slct_n = '0');
    if (seq_data = "10" or seq_data="11") then  -- write cycle, short ack cycle
      wait for 10 ns;
      ack_n <= '0';
      wait until rising_edge(clock);
      wait for 10 ns;
      ack_n <= '1';
    else                                        -- generate a longer read ack
      wait for 10 ns;
      ack_n <= '0';
      wait until rising_edge(clock);
      wait until rising_edge(clock);
      wait until rising_edge(clock);
      wait for 10 ns;
      ack_n <= '1';
    end if;
  end process ccd_ack;

end ccd12_arch;
