-- ds2433.vhd
-- dallas semiconductor ONE WIRE device with 4096 bit EEPROM memory
-- jamieson olsen <jamieson@fnal.gov>
-- 6 March 2007

-- NORMAL speed ONLY, turbo mode is not supported.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.image_pkg.all;                 -- ben cohen's image utilities

entity ds2433 is
  port(data : inout std_logic);
end ds2433;

architecture ds2433_arch of ds2433 is

  constant family_code   : std_logic_vector(7 downto 0)  := X"23";
  constant serial_number : std_logic_vector(47 downto 0) := X"000000caac92";
  constant crc_code      : std_logic_vector(7 downto 0)  := X"47";
  constant rom           : std_logic_vector(63 downto 0) := crc_code & serial_number & family_code;

  constant read_rom_opcode      : std_logic_vector(7 downto 0) := X"33";
  constant skip_rom_opcode      : std_logic_vector(7 downto 0) := X"cc";
  constant od_skip_rom_opcode   : std_logic_vector(7 downto 0) := X"3c";
  constant read_memory_opcode   : std_logic_vector(7 downto 0) := X"f0";
  constant write_scratch_opcode : std_logic_vector(7 downto 0) := X"0f";
  constant copy_scratch_opcode  : std_logic_vector(7 downto 0) := X"55";

  constant accelerator : integer := 1;

begin

  -- combinatorial stuff...

  -- sequential stuff...

  main : process

    variable ta1, ta2, es, cmd_byte, cmd_byte2 : std_logic_vector(7 downto 0);
    variable scratch_ram                       : std_logic_vector(255 downto 0);
    variable eeprom                            : std_logic_vector(4095 downto 0);
    variable eeprom_index                      : integer range 0 to 4095;
    variable overdrive                         : boolean := false;

    type     state_type is (Idle, ResetPulse, PresencePulse, Wait4Cmd, ReadRom, ReadMemory, WriteScratch, CopyScratch);
    variable state : state_type;

    procedure write_bit(d : std_logic) is  -- send a bit back to the master
    begin
      wait until data = '0';
      if (overdrive) then
        wait for 1.5 us;
      else
        wait for 4 us;
      end if;
      if (d = '1') then
        data <= 'H';
      else
        data <= '0';
      end if;
      if (overdrive) then
        wait for 6.5 us;
      else
        wait for 60 us;
      end if;
      data <= 'H';
    end procedure write_bit;

    procedure write_byte (d : std_logic_vector) is  -- send a byte back to the master
    begin
      --report "DS2433: transmitting byte " & HexImage(d) severity warning;
      for i in 0 to 7 loop
        write_bit(d(i));
      end loop;
    end procedure;

    procedure read_bit(q : out std_logic) is  -- read a bit from the master
    begin
      wait until data = '0';
      if (overdrive) then
        wait for 4 us;
      else
        wait for 40 us;
      end if;
      if (data = '0') then
        q := '0';
      else
        q := '1';
      end if;
    end procedure read_bit;

    procedure read_byte(tb : out std_logic_vector) is  -- read a byte from the master
    begin
      for i in 0 to 7 loop
        read_bit(tb(i));
      end loop;
    end procedure read_byte;

    procedure reset_pulse is  -- wait for the reset pulse and reply with an ACK pulse
    begin
      --report "DS2433: waiting for reset pulse" severity warning;        
      wait until data = '0';
      state     := ResetPulse;
      --report "DS2433: sending presence pulse" severity warning;        
      wait until (data = '1' or data = 'H' or data = 'Z');  -- does not check the length of the pulse
      state     := PresencePulse;
      wait for (60 us);
      data      <= '0';
      wait for (60 us);
      data      <= 'H';
      overdrive := false;
      --report "DS2433: done sending presence pulse waiting for command..." severity warning;        
    end procedure reset_pulse;

    procedure read_rom is               -- master reads 64 bits from the ROM
    begin
      state := ReadRom;
      --report "DS2433: sending serial number" severity warning;        
      for i in 0 to 63 loop
        write_bit(rom(i));
      end loop;
      --report "DS2433: sending serial number... done" severity warning;        
    end procedure read_rom;

    procedure read_memory is  -- master reads 4096 bits from the EEPROM LSb first
    begin
      state := ReadMemory;
      read_byte(ta1);
      read_byte(ta2);
      --report "DS2433: begin READ_MEMORY target address=" & HexImage(ta2) & HexImage(ta1) severity warning;        
      for i in 0 to 4095 loop
        write_bit(eeprom(i));
        -- report "DS2433: READ_MEMORY sending bit " & integer'image(i) & " data = " & std_logic'image(eeprom(i)) severity warning;  
      end loop;
      --report "DS2433: end READ_MEMORY" severity warning;        
    end procedure read_memory;

    procedure write_scratch is  -- master writes 16-bit target addr and 256 data bits
    begin
      state := WriteScratch;
      read_byte(ta1);
      read_byte(ta2);
      --report "DS2433: writing to scratchpad ram buffer TA2=" & HexImage(ta2) & " TA1=" & HexImage(ta1) severity warning;         
      for i in 0 to 255 loop
        read_bit(scratch_ram(i));
      end loop;
      --report "DS2433: scratchpad memory filled" severity warning;
    end procedure write_scratch;

    procedure copy_scratch is  -- save the scratchpad memory to the EEPROM
    begin
      state        := CopyScratch;
      --report "DS2433: writing scratchpad to EEPROM" severity warning;        
      read_byte(ta1);                   -- target address (in bytes)
      read_byte(ta2);  -- should fall on a 32-byte boundary (0,32,64,etc)
      read_byte(es);                    -- should be 0x1F but is not checked
      eeprom_index := (256*to_integer(unsigned(ta2))) + to_integer(unsigned(ta1));
      eeprom_index := eeprom_index*8;
      for i in 0 to 255 loop
        eeprom(eeprom_index+i) := scratch_ram(i);
      end loop;
    end procedure copy_scratch;

    procedure init_eeprom is  -- init the EEPROM contents with pseudo-random data from LFSR
      variable random_reg : std_logic_vector(32 downto 1) := X"12345678";
    begin
      --report "DS2433: init EEPROM contents (random)" severity warning;        
      for i in 0 to 4095 loop
        random_reg(32 downto 1) := random_reg(31 downto 1) & ((random_reg(32) xnor random_reg(22)) xnor (random_reg(2) xnor random_reg(1)));
        eeprom(i)               := random_reg(1);
      end loop;
    end procedure init_eeprom;

    procedure dump_eeprom is  -- format the EEPROM contents and dump to the console.
      variable bit_index   : integer range 0 to 4095;
      variable page_string : string(1 to 2);
      type     char2_array is array(31 downto 0) of string(1 to 2);
      variable byte_string : char2_array;
    begin
      --report "DS2433: EEPROM dump" severity warning;        
      pageloop : for page in 0 to 15 loop
        byteloop : for byte in 31 downto 0 loop
          bit_index         := ((page*32)+byte)*8;
          byte_string(byte) := HexImage(eeprom(bit_index+7 downto bit_index));
        end loop byteloop;
        if (page < 10) then
          page_string := "0"&integer'image(page);
        else
          page_string := integer'image(page);
        end if;
--                report "DS2433: Page " & page_string & ": " & byte_string(31) & " " & byte_string(30) & " " &
--                       byte_string(29) & " " & byte_string(28) & " " &
--                       byte_string(27) & " " & byte_string(26) & " " &
--                       byte_string(25) & " " & byte_string(24) & " " &
--                       byte_string(23) & " " & byte_string(22) & " " &
--                       byte_string(21) & " " & byte_string(20) & " " &
--                       byte_string(19) & " " & byte_string(18) & " " &
--                       byte_string(17) & " " & byte_string(16) & " " &
--                       byte_string(15) & " " & byte_string(14) & " " &
--                       byte_string(13) & " " & byte_string(12) & " " &
--                       byte_string(11) & " " & byte_string(10) & " " &
--                       byte_string( 9) & " " & byte_string( 8) & " " &
--                       byte_string( 7) & " " & byte_string( 6) & " " &
--                       byte_string( 5) & " " & byte_string( 4) & " " &
--                       byte_string( 3) & " " & byte_string( 2) & " " &
--                       byte_string( 1) & " " & byte_string( 0) & " "  severity warning;
      end loop pageloop;
    end procedure dump_eeprom;




  begin

    data  <= 'H';
    state := Idle;
    init_eeprom;
    dump_eeprom;

    main_loop : loop
      reset_pulse;
      state := Wait4Cmd;
      read_byte(cmd_byte);

      if (cmd_byte = od_skip_rom_opcode) then
        overdrive := true;
      else
        overdrive := false;
      end if;

      if (cmd_byte = read_rom_opcode) then
        read_rom;
      elsif (cmd_byte = skip_rom_opcode or cmd_byte = od_skip_rom_opcode) then
        read_byte(cmd_byte2);
        if (cmd_byte2 = read_memory_opcode) then
          read_memory;
        elsif (cmd_byte2 = write_scratch_opcode) then
          write_scratch;
        elsif (cmd_byte2 = copy_scratch_opcode) then
          copy_scratch;
          dump_eeprom;                  -- for verification
        end if;
      end if;
    end loop main_loop;

  end process;

end ds2433_arch;
