-------------------------------------------------------------------------------
-- Title      : gen_data
-- Project    : 
-------------------------------------------------------------------------------
-- File       : gen_data.vhd
-- Author     :   <otger@PWOTGER>
-- Company    : 
-- Created    : 2008-12-04
-- Last update: 2008-12-11
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: simulates slink data received from the PAN
-------------------------------------------------------------------------------
-- Copyright (c) 2008 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2008-12-04  1.0      otger    Created
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

entity gen_data is
  
  generic (
    waitTicks  : integer := 100;  -- sysclk ticks to wait between commands
    lrlClkHT   : integer := 24;         -- lrl clk half period on sysclk ticks
                                        -- units
    wait2Start : integer := 1000        -- Init wait state
    ); 

  port (
    sysclk   : in  std_logic;                      -- system clk
    reset_n  : in  std_logic;
    romDatIn : in  std_logic_vector(31 downto 0);  -- command word
    romAddr  : out std_logic_vector(15 downto 0);   -- command word address
    romRd    : out std_logic;                      -- command read
    lrl      : out std_logic_vector(3 downto 0));  -- slink lrl lines

end gen_data;
architecture arch of gen_data is

  type fsmState is (
    s_reset                             -- reset state
    , s_getData                         -- capture cmd word from rom
    , s_lrl2L                           -- pull down lrl2
    , s_setData0                        -- set correct bits to lrl lines
    , s_lrl3L                           -- pull down lrl3 (clock line)
    , s_setData1                        -- set correct bits to lrl lines
    , s_lrl3H                           -- pull high lrl3
    , s_lrl2Wait                        -- wait state before pulling high lrl2
                                        -- (end of transaction)
    , s_lrl2H                           -- pull high lrl2 (end)
    , s_waitNxt                         -- wait state between commands
    , s_error                           -- error state. If state machine gets
                                        -- here, there is an error
    );
  signal current_st, next_st : fsmState;

  signal buff2Xfer     : std_logic_vector(31 downto 0);
  signal countLoadInit : std_logic;
  signal countInitVal  : integer;
  signal countBuzz     : std_logic;
  signal lrlCycle      : integer;       -- number of LRL3 cycles (down&up)
  signal go            : std_logic;
  
  
begin  -- arch

  -----------------------------------------------------------------------------
  -- State diagram. Next state assignment
  -----------------------------------------------------------------------------

  next_st <=
    s_reset    when reset_n = '0'                               else
    s_reset    when current_st = s_reset and go = '0'           else
    s_getData  when current_st = s_reset                        else
    s_lrl2L    when current_st = s_getData                      else
    s_lrl2L    when current_st = s_lrl2L and countBuzz = '0'    else
    s_setData0 when current_st = s_lrl2L                        else
    s_lrl3L    when current_st = s_setData0                     else
    s_lrl3L    when current_st = s_lrl3L and countBuzz = '0'    else
    s_setData1 when current_st = s_lrl3L                        else
    s_lrl3H    when current_st = s_setData1                     else
    s_lrl3H    when current_st = s_lrl3H and countBuzz = '0'    else
    s_lrl2Wait when current_st = s_lrl3H and lrlCycle = 5       else
    s_setData0 when current_st = s_lrl3H                        else
    s_lrl2Wait when current_st = s_lrl2Wait and countBuzz = '0' else
    s_lrl2H    when current_st = s_lrl2Wait                     else
    s_waitNxt  when current_st = s_lrl2H                        else
    s_waitNxt  when current_st = s_waitNxt and countBuzz = '0'  else
    s_getData  when current_st = s_waitNxt                      else
    s_error;

  -----------------------------------------------------------------------------
  -- Assigning next state to current state
  -----------------------------------------------------------------------------
  p_currState : process (sysclk, reset_n)
  begin  -- process p_currState
    if reset_n = '0' then               -- asynchronous reset (active low)
      current_st <= s_reset;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      current_st <= next_st;
    end if;
  end process p_currState;

  -----------------------------------------------------------------------------
  -- Signal generation of the state machine
  --
  -- Signals:
  -- lrl(3 downto 0)
  -- countLoadInit
  -- countInitVal
  -- romAddr
  -- buff2Xfer
  -- romRd
  -- lrlCycle
  -----------------------------------------------------------------------------
  -- lrl(3)
  -- lrl(2)
  -- lrl(1) / lrl (0)
  -----------------------------------------------------------------------------
  p_lrl : process (current_st)
  begin  -- process p_lrl
    case current_st is
      when s_reset =>
        lrl(3 downto 0) <= (others => '0');
      when s_getData =>
        lrl(3 downto 0) <= (others => '1');
      when s_lrl2L =>
        lrl(3 downto 0) <= "1011";
      when s_setData0 =>
        lrl(3) <= '1';
        if lrlCycle < 5 then
          lrl(2) <= buff2Xfer(6*lrlCycle + 2);
          lrl(1) <= buff2Xfer(6*lrlCycle + 1);
          lrl(0) <= buff2Xfer(6*lrlCycle);
        else
          lrl(2) <= '0';
          lrl(1) <= buff2Xfer(31);
          lrl(0) <= buff2Xfer(30);
        end if;
      when s_lrl3L =>
        lrl(3) <= '0';
        if lrlCycle < 5 then
          lrl(2) <= buff2Xfer(6*lrlCycle + 2);
          lrl(1) <= buff2Xfer(6*lrlCycle + 1);
          lrl(0) <= buff2Xfer(6*lrlCycle);
        else
          lrl(2) <= '0';
          lrl(1) <= buff2Xfer(31);
          lrl(0) <= buff2Xfer(30);
        end if;
      when s_setData1 =>
        lrl(3) <= '0';
        if lrlCycle < 5 then
          lrl(2) <= buff2Xfer(6*lrlCycle + 5);
          lrl(1) <= buff2Xfer(6*lrlCycle + 4);
          lrl(0) <= buff2Xfer(6*lrlCycle + 3);
        else
          lrl(2 downto 0) <= "011";
        end if;
      when s_lrl3H =>
        lrl(3) <= '1';
        if lrlCycle < 5 then
          lrl(2) <= buff2Xfer(6*lrlCycle + 5);
          lrl(1) <= buff2Xfer(6*lrlCycle + 4);
          lrl(0) <= buff2Xfer(6*lrlCycle + 3);
        else
          lrl(2 downto 0) <= "011";
        end if;
      when s_lrl2Wait =>
        lrl(3)          <= '1';
        lrl(2)          <= '0';
        lrl(1 downto 0) <= "11";
      when others =>
        lrl(3 downto 0) <= (others => '1');
    end case;
  end process p_lrl;
  -----------------------------------------------------------------------------
  -- countLoadInit
  -- countInitVal
  -----------------------------------------------------------------------------
  countLoadInit <= '1' when (current_st = s_getData
                             or current_st = s_setData0
                             or current_st = s_setData1
                             or (current_st = s_lrl3H and next_st = s_lrl2Wait)
                             or (current_st = s_lrl2H and next_st = s_waitNxt))
                   else
                   '0';
  countInitVal <= waitTicks when current_st = s_lrl2H else
                  lrlClkHT;
  -----------------------------------------------------------------------------
  -- romAddr
  -----------------------------------------------------------------------------
  p_romAddr : process (sysclk, reset_n)
    variable countAddr : integer := 0;
  begin  -- process p_romAddr
    if reset_n = '0' then               -- asynchronous reset (active low)
      countAddr := 0;
      romAddr   <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if current_st = s_lrl2L and next_st = s_setData0 then
        countAddr := countAddr + 1;
      end if;
      romAddr <= std_logic_vector(to_unsigned(countAddr, 16));
    end if;
  end process p_romAddr;
  -----------------------------------------------------------------------------
  -- buff2Xfer
  -----------------------------------------------------------------------------
  p_buff2Xfer : process (sysclk, reset_n)
  begin  -- process p_buff2Xfer
    if reset_n = '0' then               -- asynchronous reset (active low)
      buff2Xfer <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if current_st = s_getData then
        buff2Xfer <= romDatIn;
      end if;
    end if;
  end process p_buff2Xfer;
  -----------------------------------------------------------------------------
  -- romRd
  -----------------------------------------------------------------------------
  romRd <= '1';
  -----------------------------------------------------------------------------
  -- lrlCycle
  -----------------------------------------------------------------------------
  p_lrlCycle : process (sysclk, reset_n)
    variable counter : integer := 0;
  begin  -- process p_lrlCycle
    if reset_n = '0' then               -- asynchronous reset (active low)
      counter  := 0;
      lrlCycle <= 0;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if current_st = s_getData then
        counter := 0;
      elsif current_st = s_lrl3H and countBuzz = '1' then
        counter := counter + 1;
      end if;

      lrlCycle <= counter;
    end if;
  end process p_lrlCycle;

  -----------------------------------------------------------------------------
  -- multi purpose countdown.
  -- When 'load' asserted, it loads 'countInitVal' into counter and starts to
  -- decrease it at each sysclk tick.

  -- load and countInitVal must be generated correctly based on current_st and
  -- next_st signals.

  -- When counter values reaches 0, it asserts countBuzz signal for one clock
  -- cycle. 
  -----------------------------------------------------------------------------
  counter : process (sysclk, reset_n)
    variable counter : integer;
  begin  -- process counter
    if reset_n = '0' then               -- asynchronous reset (active low)
      countBuzz <= '0';
      counter   := -1;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      -- generate counter value (load/ update)
      if countLoadInit = '1' then
        counter := countInitVal;
      else
        if counter >= 0 then
          counter := counter - 1;
        end if;
      end if;
      -- generate countBuzz output
      if counter = 0 then
        countBuzz <= '1';
      else
        countBuzz <= '0';
      end if;
    end if;
  end process counter;

  -----------------------------------------------------------------------------
  -- Initialize counter
  -----------------------------------------------------------------------------

  p_init: process (sysclk, reset_n)
    variable initCount : integer;
  begin  -- process p_init
    if reset_n = '0' then               -- asynchronous reset (active low)
      initCount := wait2Start;
      go <= '0';
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if initCount = 0 then
        go <= '1';
      else
        go <= '0';
        initCount := initCount - 1;
      end if;
    end if;
  end process p_init;


end arch;
